Package: Bios2cor
Type: Package
Title: From Biological Sequences and Simulations to Correlation
        Analysis
Version: 2.2.2
Date: 2024-10-15
Author: Bruck Taddese [aut],
  Antoine Garnier [aut],
  Madeline Deniaud [aut],
  Julien Pele [ctb],
  Lea Bellenger [ctb],
  Jean-Michel Becu [ctb],
  Marie Chabbert [aut, cre]
Authors@R: c(person(given = "Bruck",
                      family = "Taddese",
                      role = "aut"),
               person(given = "Antoine",
                      family = "Garnier",
                      role = "aut"),
               person(given = "Madeline",
                      family = "Deniaud",
                      role = "aut"),
               person(given = "Julien",
                      family = "Pele",
                      role = "ctb"),
               person(given = "Lea",
                      family = "Bellenger",
                      role = "ctb"),
               person(given = "Jean-Michel",
                      family = "Becu",
                      role = "ctb"),
               person(given = "Marie",
                      family = "Chabbert",
                      role = c("aut", "cre"),
                      email = "marie.chabbert@univ-angers.fr"))
Maintainer: Marie Chabbert <marie.chabbert@univ-angers.fr>
Depends: R (>= 3.5), bio3d, circular, bigmemory, parallel
Imports: igraph
Description: Utilities for computation and analysis of correlation/covariation in multiple sequence alignments and in side chain motions during molecular dynamics simulations. Features include the computation of correlation/covariation scores using a variety of scoring functions between either sequence positions in alignments or side chain dihedral angles in molecular dynamics simulations and utilities to analyze the correlation/covariation matrix through a variety of tools including network representation and principal components analysis. In addition, several utility functions are based on the R graphical environment to provide friendly tools for help in data interpretation. Examples of sequence covariation analysis are provided in: (1) Pele J, Moreau M, Abdi H, Rodien P, Castel H, Chabbert M (2014) <doi:10.1002/prot.24570> and (2) Taddese B, Deniaud M, Garnier A, Tiss A, Guissouma H, Abdi H, Henrion D, Chabbert M (2018) <doi:10.1371/journal.pcbi.1006209>. An example of side chain correlated motion analysis is provided in: Taddese B, Garnier A, Abdi H, Henrion D, Chabbert M (2020) <doi:10.1038/s41598-020-72766-1>. This work was supported by the French National Research Agency (Grant number: ANR-11-BSV2-026) and by GENCI (Grant number: 100567).
License: GPL (>= 2)
NeedsCompilation: no
Packaged: 2024-11-13 18:29:12 UTC; root
Repository: CRAN
Date/Publication: 2024-11-14 13:40:23 UTC
Built: R 4.6.0; ; 2026-04-27 15:05:40 UTC; unix

Package: Bios2cor
Type: Package
Title: From Biological Sequences and Simulations to Correlation
        Analysis
Version: 2.2.2
Date: 2024-10-15
Author: Bruck Taddese [aut],
  Antoine Garnier [aut],
  Madeline Deniaud [aut],
  Julien Pele [ctb],
  Lea Bellenger [ctb],
  Jean-Michel Becu [ctb],
  Marie Chabbert [aut, cre]
Authors@R: c(person(given = "Bruck",
                      family = "Taddese",
                      role = "aut"),
               person(given = "Antoine",
                      family = "Garnier",
                      role = "aut"),
               person(given = "Madeline",
                      family = "Deniaud",
                      role = "aut"),
               person(given = "Julien",
                      family = "Pele",
                      role = "ctb"),
               person(given = "Lea",
                      family = "Bellenger",
                      role = "ctb"),
               person(given = "Jean-Michel",
                      family = "Becu",
                      role = "ctb"),
               person(given = "Marie",
                      family = "Chabbert",
                      role = c("aut", "cre"),
                      email = "marie.chabbert@univ-angers.fr"))
Maintainer: Marie Chabbert <marie.chabbert@univ-angers.fr>
Depends: R (>= 3.5), bio3d, circular, bigmemory, parallel
Imports: igraph
Description: Utilities for computation and analysis of correlation/covariation in multiple sequence alignments and in side chain motions during molecular dynamics simulations. Features include the computation of correlation/covariation scores using a variety of scoring functions between either sequence positions in alignments or side chain dihedral angles in molecular dynamics simulations and utilities to analyze the correlation/covariation matrix through a variety of tools including network representation and principal components analysis. In addition, several utility functions are based on the R graphical environment to provide friendly tools for help in data interpretation. Examples of sequence covariation analysis are provided in: (1) Pele J, Moreau M, Abdi H, Rodien P, Castel H, Chabbert M (2014) <doi:10.1002/prot.24570> and (2) Taddese B, Deniaud M, Garnier A, Tiss A, Guissouma H, Abdi H, Henrion D, Chabbert M (2018) <doi:10.1371/journal.pcbi.1006209>. An example of side chain correlated motion analysis is provided in: Taddese B, Garnier A, Abdi H, Henrion D, Chabbert M (2020) <doi:10.1038/s41598-020-72766-1>. This work was supported by the French National Research Agency (Grant number: ANR-11-BSV2-026) and by GENCI (Grant number: 100567).
License: GPL (>= 2)
NeedsCompilation: no
Packaged: 2024-11-13 18:29:12 UTC; root
Repository: CRAN
Date/Publication: 2024-11-14 13:40:23 UTC

export("angles.plot", "angle2rotamer", "centered_pca", "top_pairs_analysis", "scores.boxplot", "write.scores", "write.entropy", "network.plot", "write.pca", "pca_screeplot", "delta_filter", "dynamic_structure", "elsc", "entropy", "scores_entropy.plot", "import.fasta", "import.msf", "mcbasc", "mi","mip", "omes", "pca_2d", "random.msa", "dynamic_circular", "dynamic_mi", "dynamic_mip", "dynamic_omes", "dynamic_entropy", "write.pca.pdb", "xyz2torsion")

import("bio3d", "bigmemory", "parallel")

importFrom("circular", "circular", "cor.circular")

importFrom("igraph", "graph_from_data_frame", "degree")

importFrom("grDevices", "adjustcolor", "dev.off", "graphics.off","pdf", "png", "rgb")
importFrom("graphics", "abline", "box", "boxplot", "layout", "lines", "par", "plot", "points","hist")
importFrom("stats", "sd", "var","na.omit")
importFrom("utils", "read.table", "setTxtProgressBar", "txtProgressBar", "write.table","write.csv")

# Package: Bios2cor 
# This file is part of the Bios2cor and Bios2mds R package.
# Bios2cor and Bios2mds are free softwares: you can redistribute them and/or modify
# them under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# See the GNU General Public License at:
# http://www.gnu.org/licenses/
#

random.msa <- function (nb.seq = 100, id = "SEQ", nb.pos = 100, gap = FALSE, aa.strict = FALSE,align = NULL, align.replace = TRUE) {

  #one letter codes for amino acids 
  aa <- c("A", "R", "N", "D", "C", "Q", "E", "G", "H", "I", "L", "K",
    "M", "F", "P", "S", "T", "W", "Y", "V", "B", "Z", "J", "X")
  replace <- TRUE
  #remove ambiguous amino acids
  if (aa.strict)
    aa <- aa[1:20]

  if (gap)
    aa <- c(aa, "-")

  if(!is.null(align)){
    if (!inherits(align, "align"))
        stop("mmds is not a 'align' object")
    aa <-as.vector(unlist(align))
    replace<-align.replace
  }

  msa <- lapply(seq_len(nb.seq), function (i) {sample(aa, nb.pos, replace = replace)})

  msa.names <- paste(id, seq_len(nb.seq), sep = "")

  names(msa) <- msa.names

  return(msa)
}

# Package: Bios2cor 
# This file is part of Bios2cor R package.
# Bios2cor is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# See the GNU General Public License at:
# http://www.gnu.org/licenses/
#
scores_entropy.plot <- function(entropy, corr_matrix, filepathroot, elite=25, high=275, filter=NULL){


  if(missing(entropy)) 
    stop("An 'entropy' object is required") 
  

  if(missing(corr_matrix)) 
    stop("A correlation matrix is required") 

  if(!is.matrix(corr_matrix)) 
    stop("corr_matrix must be a matrix. Select a score or Zscore matrix") 

  corr_score <- corr_matrix
  names_c <- colnames(corr_matrix)
  size_matrix <- length(names_c)
  names_s  <- names(entropy)


  if(!is.null(filter)){
    for(i in 1:size_matrix){
      for(j in 1:size_matrix){
	name_i <- names_c[i]
	name_j <- names_c[j]
	ponderation_i <- filter[name_i]
	ponderation_j <- filter[name_j]
	corr_score[i,j] <- corr_score[i,j]*ponderation_i*ponderation_j
      }
    }
  }
  
# Checking the numbering used in the entropy object and in the correlation matrix 
  numbering1 <- grep(".", names_s, fixed= TRUE)
  numbering2 <- grep(".", names_c, fixed= TRUE)
	
# Analysis is possible only if the same numbering is used for entropy and correlation
  if(((length(numbering1) == 0) && (length(numbering2) == 0)) || ((length(numbering1) != 0) && (length(numbering2) != 0))){

     entropy_i <- c()
     entropy_j <- c()
     score <- c()

      k <-0
      for(i in 1:(size_matrix-1)){
         for(j in (i+1):size_matrix){
	  posi <- names_c[i]
	  posj <- names_c[j]
          #find entropy for each position of the pair
	  if(posi %in% names_s && posj %in% names_s){
	     k <- k+1
	     entropy_i[k] <- entropy[posi]
	     entropy_j[k] <- entropy[posj]
	     score[k] <- corr_score[posi, posj]
	  }else{
	     next
	  }
        }
      }


results <- cbind(score,entropy_i,entropy_j)
result_ordered <- results[order(as.numeric(results[,1]), decreasing = TRUE),]

x<- result_ordered[,1]
i<-result_ordered[,2]
j<-result_ordered[,3]

nb_pairs <-length(x)

bottom_elite <- nb_pairs - elite
bottom_high <- nb_pairs - high
stop_high <- bottom_elite - 1
start_high <- elite + 1

if (is.null(filepathroot)){
   filename <- paste(tempdir(), "/ei_ej.pdf", sep ="")
} else {
filename = paste(filepathroot, "_ei_ej.pdf", sep ="")
}

pdf(file= filename)

plot(i, j, main= filepathroot, xlab= "S[i]", ylab= "S[j]", pch= ".", col= "gray80",cex.axis=1.4, cex.lab=1.4)
if(is.null(filter)) {
points(i[bottom_high:stop_high], j[bottom_high:stop_high], col = "lightpink1", pch=20)
points(i[bottom_elite:nb_pairs], j[bottom_elite:nb_pairs], col = "red", pch=20)
points(i[start_high:high], j[start_high:high], col = "skyblue2", pch=20)
points(i[1:elite], j[1:elite], col = "blue", pch=20)
} else {
points(i[start_high:high], j[start_high:high], col = "skyblue2", pch=20)
points(i[1:elite], j[1:elite], col = "blue", pch=20)
}

dev.off()

  } else {
 warning("Mismatch in the notation used for the correlation and entropy files. Please verify your files!") 
 }
}

# Package: Bios2cor 
# This file is part of Bios2cor R package.
# Bios2cor is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# See the GNU General Public License at:
# http://www.gnu.org/licenses/
#

entropy <- function (align, gap_ratio = 0.2) {

    if ((gap_ratio < 0) | (gap_ratio > 1)) {
        stop("Error in entropy argument: gap_ratio must be in the [0,1] range.")
    }

    MSA <- matrix(as.vector(unlist(align)), ncol= length(align[[1]]), byrow= TRUE)
    nb_pos <- length(MSA[1,]) #number of positions in the alignment
    nb_seq <- length(MSA[,1]) #number of sequences in the alignment
    colnames(MSA)<-c(1:nb_pos)

    gap <- 1-gap_ratio      #gap value indicates the minimal ratio of aa to nb_seq in the MSA 
    if (gap < 1/nb_seq) {
      gap <- 1/nb_seq     # positions must have at leat ONE aa to be taken into account (removes gap column)
    }
    
    names<-c("A","C","D","E","F","G","H","I","K","L","M","N","P","Q","R","S","T","V","W","Y","-")

    # Binary matrix indicating which amino acid is present or not at position i in the sequence j
    AA<-lapply(1:nb_pos, function(i){
	    t(table(c(MSA[,i],names),row.names=c(1:(nb_seq+21))))[1:nb_seq, -1]
    })
    
    names<-c("A","C","D","E","F","G","H","I","K","L","M","N","P","Q","R","S","T","V","W","Y")
    nb_aa <- length(names)

    entropy <- matrix(0, ncol= nb_pos, nrow= 1)

     # Calculating validity and entropy for each  position
    entropy <- unlist(lapply(1:nb_pos, function(i){
       mat_i <- AA[[i]] #matrix nb_seq*nb_aa
        S_i <- colSums(AA[[i]])
        Tot_i <- sum(S_i)
        if (Tot_i/nb_seq >= gap) {
      		percent_i <- S_i/Tot_i
      		entropy_i <- 0
      		for(j in 1:nb_aa){
         		if (S_i[[j]] == 0) {
				entropy_i <- entropy_i
	 		} else {
				entropy_i <- entropy_i - percent_i[[j]]*log(percent_i[[j]], base = 20) 
         		}
      		}	
      		entropy[i] <- entropy_i
    	} else {
		entropy[i] <- ''	
    	}
   })) 	
          
	names(entropy) <- c(1:nb_pos)
	return(entropy)
 

}

# Bios2cor is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# See the GNU General Public License at:
# http://www.gnu.org/licenses/
#
pca_screeplot <- function(pca_struct, filepathroot){
  pca_coord <- pca_struct$coord
  
  nb_component <- length(pca_coord[1,])
  pca_positions <- rownames(pca_coord)

  components <- 1:nb_component
  
  variances <- unlist(lapply(1:nb_component, function(x){var(pca_coord[,x])}))

  if(is.null(filepathroot)) {
    filename <- paste(tempdir(), "/SCREEPLOT.png", sep="")
  } else {
    filename <- paste(filepathroot, "_SCREEPLOT.png", sep="")
  }  
  
  png(filename, width = 600, height = 600, units = "px")
    plot(components, variances, main= basename(filename))
    lines(components, variances)
  dev.off()
}

 

# Package: Bios2cor 
# This file is part of Bios2cor R package.
# Bios2cor is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# See the GNU General Public License at:
# http://www.gnu.org/licenses/
#
write.pca.pdb <- function(corr_pca, filepathroot, trio_comp= c(1:3)){

  if (missing(corr_pca)) {
      stop("A PCA object created by the centered_pca function is required")
  }

  pca_coord <- corr_pca$coord
  pca_1 <- trio_comp[1]
  pca_2 <- trio_comp[2]
  pca_3 <- trio_comp[3]
  
  if (is.null(filepathroot)) {
    pdb_filepath <- paste(tempdir(), "/PCA_", pca_1, "_",pca_2, "_",pca_3, ".pdb", sep ="")
    pml_filepath <- paste(tempdir(), "/PCA_", pca_1, "_",pca_2, "_",pca_3, ".pml", sep ="")   
  } else {
    pdb_filepath <- paste(filepathroot, "_PCA_", pca_1, "_",pca_2, "_",pca_3, ".pdb", sep ="")
    pml_filepath <- paste(filepathroot, "_PCA_", pca_1, "_",pca_2, "_",pca_3, ".pml", sep ="")   
  }


  pca_positions <- rownames(pca_coord)
  pca_size <- length(pca_positions)


#check whether data are from a MSA or a trajectory
 numbering <- grep(".", pca_positions, fixed= TRUE)

  alpha <- c("A","B","C","D")
  colors <- c("blue", "red", "green", "orange", "white")
  background_color <- paste("bg_color", colors[5])
  write(background_color, file= pml_filepath, append= FALSE)
  for(i in 1:4){
      chain_coloration <- paste("color", colors[i], ", chain", alpha[i])
      write(chain_coloration, file= pml_filepath, append= TRUE)
  }



#Configuring elements
  as_sphere <- "as sphere"
  sphere_scale <- "set sphere_scale, 1"
  write(as_sphere, file= pml_filepath, append= TRUE)
  write(sphere_scale, file= pml_filepath, append= TRUE)




## Writing PDB file

  pdb_line <- "REMARK PDB file created for visualization of PCA analysis"
  write(pdb_line, file= pdb_filepath, append= FALSE)
 
##fields required in each line
  #fields without change
  head <- "HETATM"
  atom_name <- "O"
  res_type <- "HOH"
  occupation <- "1.00"
  B_factor <- 100.00
  atom_type <- "O"
  format <- "%6s%5s  %-4s%3s %1s%4s    %8s%8s%8s   %4s%6s     %4s"  

  #fields with changes 

  if(length(numbering) == 0){

   for(pos in 1:pca_size){
      x <- pca_coord[pos, pca_1]
      y <- pca_coord[pos, pca_2]
      z <- pca_coord[pos, pca_3]

     if (!is.na(x) & !is.na(y) & !is.na(z)) { 
      residue_number <- pca_positions[pos]
      chain_ID <- "A"
      pdb_line <- sprintf(format, head, pos, atom_name, res_type, chain_ID, residue_number, round(x,digits=3), round(y,digits=3), round(z,digits=3), occupation, B_factor, atom_type)
      write(pdb_line, file= pdb_filepath, append= TRUE)
     }
    }
   last_number <- as.numeric(residue_number)

  }else{

  resno <- sub("\\..*$", "", pca_positions)
  angle <- sub("^[0-9]*\\.","", pca_positions)

  for(pos in 1:pca_size){
      x <- pca_coord[pos, pca_1]
      y <- pca_coord[pos, pca_2]
      z <- pca_coord[pos, pca_3]

    if (!is.na(x) & !is.na(y) & !is.na(z)) { 
    residue_number <- resno[pos]
    angle_ID <- angle[pos]

    if (angle_ID == "chi1") {chain_ID <- "A"}
    if (angle_ID == "chi2") {chain_ID <- "B"}
    if (angle_ID == "chi3") {chain_ID <- "C"}
    if (angle_ID == "chi4") {chain_ID <- "D"}

    pdb_line <- sprintf(format, head, pos, atom_name, res_type, chain_ID, residue_number, round(x,digits=3), round(y,digits=3), round(z,digits=3), occupation, B_factor, atom_type)
    
    write(pdb_line, file= pdb_filepath, append= TRUE)
   }
  }
    last_number <- as.numeric(residue_number)


}
  
  #Creating elements used to create axis
  max_x <- max(abs(na.omit(pca_coord[,1])))
  norm <- round(max_x*2, digits=3)
  axis_fake_atoms <- c("O","C","N", "Pb")
  axis_fake_chain_ID <- "Z"

  x_pos <- pca_size+1
  y_pos <- pca_size+2
  z_pos <- pca_size+3
  origin_pos <- pca_size+4

  x_residue <- last_number+1
  y_residue <- last_number+2
  z_residue <- last_number+3
  origin_residue <- last_number+4
 
  x_name <- "XXX"
  y_name <- "YYY"
  z_name <- "ZZZ"
  origin_name <- "000"
  
  x_axis <- sprintf(format, head, x_residue, axis_fake_atoms[1], x_name, axis_fake_chain_ID, x_residue, norm, 0, 0, occupation, B_factor, axis_fake_atoms[1])
  y_axis <- sprintf(format, head, y_residue, axis_fake_atoms[2], y_name, axis_fake_chain_ID, y_residue, 0, norm, 0, occupation, B_factor, axis_fake_atoms[2])
  z_axis <- sprintf(format, head, z_residue, axis_fake_atoms[3], z_name, axis_fake_chain_ID, z_residue, 0, 0, norm, occupation, B_factor, axis_fake_atoms[3])
  origin <- sprintf(format, head, origin_residue, axis_fake_atoms[4], origin_name, axis_fake_chain_ID, origin_residue, 0, 0, 0, occupation, B_factor, axis_fake_atoms[4])
  
  write(x_axis, file= pdb_filepath, append= TRUE)
  write(y_axis, file= pdb_filepath, append= TRUE)
  write(z_axis, file= pdb_filepath, append= TRUE)
  write(origin, file= pdb_filepath, append= TRUE)
  
  #Connecting x, y and z to origin
  connect_x <- paste("CONECT", x_residue, origin_residue)
  connect_y <- paste("CONECT", y_residue, origin_residue)
  connect_z <- paste("CONECT", z_residue, origin_residue)
  
  write(connect_x, file= pdb_filepath, append= TRUE)
  write(connect_y, file= pdb_filepath, append= TRUE)
  write(connect_z, file= pdb_filepath, append= TRUE)
  write("END", file= pdb_filepath, append= TRUE)

  
  #Display CONECT elements when loading pml file
  enable_connections <- "show lines"
  write(enable_connections, file= pml_filepath, append= TRUE)
  hidden_spheres <- "hide spheres, chain Z" 
  write(hidden_spheres, file= pml_filepath, append= TRUE)
 
}

# Bios2cor is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# See the GNU General Public License at:
# http://www.gnu.org/licenses/
#
write.pca <- function(corr_pca, filepathroot, pc= NULL, entropy= NULL){
  
  if (missing(corr_pca)) {
      stop("A PCA object created by the centered_pca function is required")
  }

 if (is.null(filepathroot)) {
  filename <- paste(tempdir(), "/PCA_COORD.csv", sep="")
  }else{
  filename <-paste(filepathroot, "_PCA_COORD.csv", sep="")
  }


  pca_coord <- corr_pca$coord
  pca_positions <- rownames(pca_coord)
  pca_size <- length(pca_coord[,1])
  
  if (is.null(pc)) { 
  pca_dim <- length(pca_coord[1,])
  } else {
  pca_dim <- pc
  }

  head <- "position"
  if(!is.null(entropy)) {
    head <- paste(head, "entropy")
  }
  
  lapply(1:pca_dim, function(dim){
    head <<- paste(head, paste("PCA", dim, sep= ""))
  })
  
  write(head, file= filename, append= FALSE)
  
  if(!is.null(entropy)) {
    for(pos in 1:pca_size){
      pos_line <- pca_coord[pos,]
      position <- pca_positions[pos]
      entropy_val <- format(as.numeric(entropy[position]), digits=3, nsmall=3)
      
      #Ignoring possible NaN values
      if(sum(is.na(pos_line)) <= 0){
	coord_tmp <- paste(pca_coord[pos, 1:pca_dim], collapse= " ")
	current_line <- paste(position, entropy_val, coord_tmp)
	write(current_line, file= filename, append= TRUE)
      }
    }
  } else {
    for(pos in 1:pca_size){
      pos_line <- pca_coord[pos,]
      position <- pca_positions[pos]
      
      #Ignoring possible NaN values
      if(sum(is.na(pos_line)) <= 0){
	coords <- ""
	for(dim in 1:pca_dim){
	  coords <- paste(coords, pca_coord[pos,dim])
	}
	current_line <- paste(position, coords, sep= "")
	write(current_line, file= filename, append= TRUE)
      }
    }
  }
  
}

 

# Bios2cor is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# See the GNU General Public License at:
# http://www.gnu.org/licenses/
#
elsc <- function (align, gap_ratio = 0.2){

  if ((gap_ratio < 0) | (gap_ratio > 1)) {
    stop("Error in elsc argument: gap_ratio must be in the [0,1] range")
  }

    diag <- 0

    msa<-align
    MSA <- matrix(as.vector(unlist(msa)), ncol = length(msa[[1]]),byrow = TRUE)
    nb_pos <- length(MSA[1,]) #number of positions in the alignment
    nb_seq <- length(MSA[,1]) #number of sequences in the alignment
    colnames(MSA)<-c(1:nb_pos)
    pos_names <- colnames(MSA)  

    gap <- 1-gap_ratio      #gap value indicates the minimal ratio of aa to nb_seq in the MSA 
    if (gap < 1/nb_seq) {
      gap <- 1/nb_seq     # positions must have at leat ONE aa to be taken into account (removes gap column)
    }

    names<-c("A","C","D","E","F","G","H","I","K","L","M","N","P","Q","R","S","T","V","W","Y","-")

    AA<-lapply(1:nb_pos,function(i){
	t(table(c(MSA[,i],names),row.names=c(1:(nb_seq+21))))[1:nb_seq, -1]
    })

    names<-c("A","C","D","E","F","G","H","I","K","L","M","N","P","Q","R","S","T","V","W","Y")
    nb_aa <- length(names)
	
    COV2<-matrix(0, ncol= nb_pos, nrow= nb_pos)
	
    # Setting columns and rows names before matrix reduction
    rownames(COV2)<-pos_names
    colnames(COV2)<-pos_names
	
    # Determining valid positions with gap ratio under the (1 - gap) limit 
    Valid_pos <- c()
    for(i in 1:nb_pos){
       mat_i <- AA[[i]] #matrix nb_seq*nb_aa
        S_i <- colSums(AA[[i]])
        Tot_i <- sum(S_i)
        if (Tot_i/nb_seq >= gap) {
           Valid_pos <- c(Valid_pos, i)
        }
    }
    nb_Valid_pos <- length(Valid_pos)
	
    # Calculating ELSC score for each valid position
    for(i in 1:nb_Valid_pos){
        pos_i <- Valid_pos[i] # current valid position
        cat(paste("pos_i : ", pos_i, "\n"))
	for(j in i:nb_Valid_pos){
	    pos_j <- Valid_pos[j]
		
	    aln_tot_i<-AA[[pos_i]]
	    aln_tot_i[is.na(aln_tot_i)] <- 0
		
	    aln_tot_j<-AA[[pos_j]]
	    aln_tot_j[is.na(aln_tot_j)] <- 0
		
	    Nyj<-colSums(aln_tot_j) # number of amino acids y in position j in the full alignment
	    v<-sort(colSums(aln_tot_i),decreasing = TRUE)
	    v<-unique(c(which(aln_tot_i[,names(v[1])] == 1))) # sequences where the most occuring amino acid in position i appears 

	    ss_aln_i<-aln_tot_i[v,]
	    nb_seq_ss_aln_i <- length(ss_aln_i[,1])
		
	    ss_aln_j<-aln_tot_j[v,]
	    nb_seq_ss_aln_j <- length(ss_aln_j[,1])
		
	    n<-colSums(ss_aln_j) # number of amino acids y in position j in the sub-alignment
	    mf<-(Nyj/nb_seq)*nb_seq_ss_aln_j
		
	    calc<-matrix(0, ncol=2, nrow= nb_aa, dimnames = list(names,c("r", "m")))
	    calc[,2]<-trunc(mf) # inferior mf round
	    calc[,1]<-(mf-calc[,2]) # remainder

	    calc<-calc[order(calc[,1],rownames(calc), decreasing =T),] # order m by decreasing order of r then by alpha inverse order of aa
		
	    # The sum of the "m" column must be equal to the sum of "n"
	    if(sum(calc[,2]) > sum(n)){
		z<-nb_aa
		while((sum(calc[,2]))!=(sum(n))){ 
		    calc[z,2]<-(calc[z,2]-1)
		    if(z == 1) z <- nb_aa else z<-z-1
		}
	    } else {
		z<-1
		while((sum(calc[,2]))!=(sum(n))){
		    calc[z,2]<-(calc[z,2]+1)
		    if(z == nb_aa) z <- 1 else z<-z+1
		}
	    }
		
	    m<-calc[order(rownames(calc)),2]
	    COV2[pos_i, pos_j]<-(-log(prod(choose(Nyj, n)/choose(Nyj, m)))) #matrice asymetrique - vrais scores obtenus 
	    }
	}



	#Complete matrix second triangular part
	COV2 <- COV2 + t(COV2)
	diag(COV2) <- diag

	#Reducting the final correlation matrix to the valid positions
	COV2 <- COV2[Valid_pos,]
	COV2 <- COV2[,Valid_pos]

	#Removing "Inf" and "NaN" values
	COV2[is.infinite(COV2)] <- 0
	COV2[is.na(COV2)] <- 0
	

    res <- list()

    # save matrix of score 
    res$score <- COV2
    
    # compute and save matrix of Z_scores
    # mean and stdev must be calculated on off diagonal elements  
    mean_up <- mean(COV2[upper.tri(COV2)])
    stdev <- sd(COV2[upper.tri(COV2)])
    
    COV2 <- (COV2-mean_up)/stdev
    diag(COV2) <- diag    
    res$Zscore <- COV2

   return(res)


}

import.fasta <- function (file, aa.to.upper = TRUE, gap.to.dash = TRUE) {

  if(missing(file)) {
    stop("file is missing")
  }

  # Read as a vector of lines
  lines <- readLines(file)

  # Localize sequence identifiers and check fasta format
  loc <- grep(">", lines)
  if (length(loc) == 0){
        stop("file is not in fasta format")
  }
  # Get sequence identifiers
  id <- sub("^>(\\S+).*$","\\1", lines[loc])
  nb.seq <- length(id)

  # Localize sequence pieces for each identifier
  start <- loc + 1
  end <- loc - 1  
  end <- c(end[-1], length(lines))

  seq <- sapply(seq_len(nb.seq), function(i) {paste(lines[start[i]:end[i]], collapse = "")})
  seq <- gsub("\\s", "", seq)

  # Turn aa into upper case
  if (aa.to.upper)
    seq <- toupper(seq)

  # Give a list of split sequences
  seq <- strsplit(seq, split = "")
  names(seq) <- id

  # Turn gap into dash character
  if (gap.to.dash)
    seq <- lapply(seq, function (i) {i[is.gap(i)] <- "-"; return(i)})
  class (seq) <- c("align")
  return(seq)
}

# Package: Bios2cor 
# This file is part of Bios2cor R package.
# Bios2cor is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# See the GNU General Public License at:
# http://www.gnu.org/licenses/
#
mcbasc <- function (align, gap_ratio = 0.2) {

    if ((gap_ratio < 0) | (gap_ratio > 1)) {
      stop("gap_ratio must be in the [0,1] range.")
    }

    diag <- 0	

    msa<-align
    MSA <- matrix(as.vector(unlist(msa)), ncol= length(msa[[1]]), byrow= TRUE)
    nb_pos <- length(MSA[1,]) #number of positions in the alignment
    nb_seq <- length(MSA[,1]) #number of sequences in the alignment
    colnames(MSA)<-c(1:nb_pos)
    pos_names <- colnames(MSA)
	
    gap <- 1-gap_ratio      #gap value indicates the minimal ratio of aa to nb_seq in the MSA 
    if (gap < 1/nb_seq) {
      gap <- 1/nb_seq     # positions must have at leat ONE aa to be taken into account (removes gap column)
    }


    names<-c("A","C","D","E","F","G","H","I","K","L","M","N","P","Q","R","S","T","V","W","Y","-")

    # Binary matrix indicating which amino acid is present or not at position i in the sequence j
    AA<-lapply(1:nb_pos, function(i){
	    t(table(c(MSA[,i],names),row.names=c(1:(nb_seq+21))))[1:nb_seq, -1]
    })
    
    names<-c("A","C","D","E","F","G","H","I","K","L","M","N","P","Q","R","S","T","V","W","Y")
    nb_aa <- length(names)

    # Determining valid positions with correct gap ratio (equal to 1 - gap argument)
    Valid_pos <- c()
    for(i in 1:nb_pos){
       mat_i <- AA[[i]] #matrix nb_seq*nb_aa
        S_i <- colSums(AA[[i]])
        Tot_i <- sum(S_i)
        if (Tot_i/nb_seq >= gap) {
           Valid_pos <- c(Valid_pos, i)
        }
    }
    nb_Valid_pos <- length(Valid_pos)




    matrice_mclachlan <- matrix(c(
			8,1,3,4,1,3,3,2,3,2,3,3,4,3,2,4,3,3,1,1,		
			1,9,1,0,0,1,3,1,0,0,3,1,0,0,1,2,2,1,2,1,		
			3,1,8,5,1,3,4,0,3,1,2,5,3,4,1,3,3,1,0,1,		
			4,0,5,8,0,3,2,1,4,1,1,4,4,5,3,4,4,2,1,2,		
			1,0,1,0,9,0,4,3,0,5,5,0,1,0,1,2,1,3,6,6,		
			3,1,3,3,0,8,2,1,3,1,1,3,3,2,3,3,2,2,1,0,		
			3,3,4,2,4,2,8,2,4,2,3,4,3,4,5,3,4,2,3,4,		
			2,1,0,1,3,1,2,8,1,5,5,1,1,0,1,2,3,5,3,3,		
			3,0,3,4,0,3,4,1,8,2,1,4,3,4,5,3,3,2,1,1,		
			2,0,1,1,5,1,2,5,2,8,6,1,1,3,2,2,3,5,3,3,		
			3,3,2,1,5,1,3,5,1,6,8,2,1,3,1,2,3,4,1,2,		
			3,1,5,4,0,3,4,1,4,1,2,8,1,4,3,5,3,1,0,2,		
			4,0,3,4,1,3,3,1,3,1,1,1,8,3,3,3,3,2,0,0,		 
			3,0,4,5,0,2,4,0,4,3,3,4,3,8,5,4,3,2,2,1,		
			2,1,1,3,1,3,5,1,5,2,1,3,3,5,8,4,3,2,3,2,		
			4,2,3,4,2,3,3,2,3,2,2,5,3,4,4,8,5,2,3,3,		
			3,2,3,4,1,2,4,3,3,3,3,3,3,3,3,5,8,3,2,1,		
			3,1,1,2,3,2,2,5,2,5,4,1,2,2,2,2,3,8,2,3,		
			1,2,0,1,6,1,3,3,1,3,1,0,0,2,3,3,2,2,9,6,		
			1,1,1,2,6,0,4,3,1,3,2,2,0,1,2,3,1,3,6,9),nrow = 20)
	
    names<-c("A","C","D","E","F","G","H","I","K","L","M","N","P","Q","R","S","T","V","W","Y")
    rownames(matrice_mclachlan) <- names
    colnames(matrice_mclachlan) <- names

    A<-lapply(1:nb_pos, function(i){
	  matrix(0, ncol=nb_seq, nrow=nb_seq, dimnames = list(c(MSA[,i]), c(MSA[,i])))
    })

    for (k in 1:nb_pos) {
 	mat<-A[[k]]
	for (i in names) {
		for (j in names) {
			mat[rownames(mat)==i, colnames(mat)==j] <- matrice_mclachlan[rownames(matrice_mclachlan)==i, colnames(matrice_mclachlan)==j]
		}
	}
	A[[k]] <- mat
    }

    COV2<-matrix(0, ncol= nb_pos, nrow= nb_pos)
	
    # Setting columns and rows names before matrix reduction
    rownames(COV2)<-pos_names
    colnames(COV2)<-pos_names
	
    # Calculating McBASC score for each valid position
    for(i in 1:nb_Valid_pos){
	pos_i <- Valid_pos[i] #current valid position
	mat_i <- A[[pos_i]] #matrix nb_seq*nb_aa
	    
	cat(paste("pos_i : ", pos_i, "\n"))
	    
 	for(j in i:nb_Valid_pos){
	    pos_j <- Valid_pos[j]
	    mat_j <- A[[pos_j]] #matrix nb_seq*nb_aa
		
	    SUM<-sum((mat_i-mean(mat_i))*(mat_j-mean(mat_j)))
	    COV2[pos_i, pos_j] <- (1/((nb_seq)^2))*(SUM/(sd(mat_i)*sd(mat_j)))
	}
    }


    # Complete the second triangular part of the matrix	
    COV2 <- COV2+t(COV2)
    diag(COV2) <- diag
	
    # Reducting the final correlation matrix to the valid positions
    COV2 <- COV2[Valid_pos,]
    COV2 <- COV2[,Valid_pos]
	
    # Removing "Inf" and "NaN" values
    COV2[is.infinite(COV2)] <- 0
    COV2[is.na(COV2)] <- 0
	
    res <- list()
    res$score <- COV2
	
    # Compute and save matrix of Z_scores
    # Mean and stdev must be calculated on off diagonal elements  
    mean_up <- mean(COV2[upper.tri(COV2)])
    stdev <- sd(COV2[upper.tri(COV2)])
    COV2 <- (COV2-mean_up)/stdev
    diag(COV2) <- diag
    res$Zscore <- COV2

    return(res)


}

# Package: Bios2cor 
# This file is part of Bios2cor R package.
# Bios2cor is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# See the GNU General Public License at:
# http://www.gnu.org/licenses/
#
dynamic_mip <- function(dynamic_structure, rotamers, res_selection= c("C","I","L","M","V","R","H","K","D","E","N","Q","F","Y","W","T","S","P")){

  if(missing(dynamic_structure)){
    stop("An object of class 'structure' is required as first argument")
  }

  if (missing(rotamers)){
    stop("A matrix of type 'rotamers' is required as second argument")
  }


  nb_angles <- length(rotamers[1,])
  nb_frames <- length(rotamers[,1])
  
  pos_names <- colnames(rotamers)
  
  names <- unique(as.vector(rotamers))
  nb_rotamers <- length(names)
  
  # Binary matrix indicating which rotamer is present or not at dihedral position i for each trajectory frame 
  ROT<-lapply(1:nb_angles, function(i){
      t(table(c(rotamers[,i], names), row.names=c(1:(nb_frames+nb_rotamers))))[1:nb_frames,]
  })
  
  MI <- matrix(0, ncol= nb_angles, nrow= nb_angles)
	
  freq_ij <- matrix(0, ncol= nb_rotamers, nrow= nb_rotamers, dimnames=list(names, names))
  
  # Calculating MI score for each position
  for(i in 1:nb_angles){
      mat_i <- ROT[[i]] # matrix nb_frames*nb_rotamers
      
      cat(paste("i : ", i, "\n"))
      
      # Rotamer frequency at dihedral position i in the trajectory
      freq_i <- colSums(mat_i)/nb_frames
      
      for(j in i:nb_angles){
	  mat_j <- ROT[[j]] #matrix nb_seq*nb_rotamers
	  
	  # Rotamer frequency at dihedral position j in the trajectory
	  freq_j <- colSums(mat_j)/nb_frames #matrix 1*nb_rotamers

	  # Rotamer frequency at dihedral positions i AND j in the trajectory
	  freq_p <- ((t(mat_i))%*%mat_j)/nb_frames #matrix nb_rotamers*nb_rotamers
	  
	  for (k in names) {
	      for (l in names) {
		  freq_ij[k,l]<-freq_i[k]*freq_j[l]
	      }
	  }
	  
	  LOG<-(log(freq_p, base=9)-log(freq_ij, base=9))
	  LOG<-replace(LOG, which(LOG=="-Inf"),0)
	  LOG<-replace(LOG, which(LOG=="Inf"),0)
	  LOG<-replace(LOG, which(LOG=="NaN"),0)
	  
	  MI[i, j]<-sum((freq_p)*(LOG))
      }
  }
  
  # Product correction P
  P <- matrix(0, ncol= nb_angles, nrow= nb_angles)
  
  mean_cov=sum(MI)/(length(which(MI!=0)))
  for(i in 1:(nb_angles-1)){
	  mean_i<-(sum(MI[i,])+sum(MI[,i]))/(nb_angles-1)

	  for(k in (i+1):nb_angles){
		  mean_k<-(sum(MI[k,])+sum(MI[,k]))/(nb_angles-1)
		  P[i, k]<-((mean_i*mean_k))/(mean_cov)
	  }
  }

  diag(P)<-0

  # MIP final value
  COV2<-matrix(0, ncol= nb_angles, nrow= nb_angles)
  COV2 <- MI-P 
  
  # Setting columns and rows names before matrix reduction
  rownames(COV2)<-paste(pos_names, sep="")
  colnames(COV2)<-paste(pos_names, sep="")
  
  COV2<-COV2+t(COV2) #Complete the second triangular part of the matrix
  diag(COV2) <- 0
  
    
  #Removing "Inf" and "NA" values
  COV2[is.infinite(COV2)] <- 0
  COV2[is.na(COV2)] <- 0
 
  selected_pos_names <- colnames(COV2)
  
  if(!is.null(res_selection)){
    
    #Position selection
    tor.seq <- dynamic_structure$tor.seq
    
    residue_selection.inds <- which(tor.seq %in% res_selection)
    COV2 <- COV2[, residue_selection.inds]
    COV2 <- COV2[residue_selection.inds, ]
    
    nb_angles <- length(COV2[1,])
    selected_pos_names <- colnames(COV2)
  }
  
 
res <- list()
    # Save matrix of scores 
    res$score <- COV2
    
    # Compute and save matrix of Z_scores
    # Mean and stdev must be calculated on off diagonal elements  
    mean_up <- mean(COV2[upper.tri(COV2)])
    stdev <- sd(COV2[upper.tri(COV2)])
    COV3 <- (COV2-mean_up)/stdev
    diag(COV3) <- 0
    res$Zscore <- COV3

    # Save matrices of scores and Zscores without auto correlation
    # Create and save matrix with 0 values for autocorrelation (dihedral angles within the same residue)
    COV5 <- COV2
    res_num <- c()
    res_num <- unlist(lapply(selected_pos_names, function(x){strsplit(x, "[.]")[[1]][1]}))

    # Create matrix with NA that will be used for Zscores 
    for(i in 1:nb_angles){
      for(j in i:nb_angles){
	res_i <- res_num[i]
	res_j <- res_num[j]
	
	if(res_i == res_j){
	  COV5[i,j] <- NA
	  COV5[j,i] <- NA
	}
      }
    }
 

    # Save matrix of score with 0 for autocorrelation  
    COV4 <-COV5
    COV4[is.na(COV4)] <- 0
    res$score_noauto <- COV4

    # Compute and save matrix of Z_scores without auto correlation  
    # Mean and stdev are calculated on non NA elements of upper triangle 
    mean_noauto <- mean(COV5[upper.tri(COV5)],na.rm=TRUE)
    stdev_noauto <- sd(COV5[upper.tri(COV5)],na.rm=TRUE)
    
    COV5[is.na(COV5)] <- 0
    COV5 <- (COV5-mean_noauto)/stdev_noauto

    # Correct diag and autocorrelated elements	
    diag(COV5) <- 0
    
    for(i in 1:nb_angles){
      for(j in i:nb_angles){
	res_i <- res_num[i]
	res_j <- res_num[j]
	
	if(res_i == res_j){
	  COV5[i,j] <- 0
	  COV5[j,i] <- 0
	}
      }
    }
  
    res$Zscore_noauto <- COV5

return(res)
}
 

# Package: Bios2cor 
# This file is part of Bios2cor R package.
# Bios2cor is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# See the GNU General Public License at:
# http://www.gnu.org/licenses/

mi <- function (align, gap_ratio = 0.2) {

    if ((gap_ratio < 0) | (gap_ratio > 1)) {
      stop("gap_ratio must be in the [0,1] range.")
    }

    diag <- 0	

    msa<-align
    MSA <- matrix(as.vector(unlist(msa)), ncol= length(msa[[1]]), byrow= TRUE)
    nb_pos <- length(MSA[1,]) #number of positions in the alignment
    nb_seq <- length(MSA[,1]) #number of sequences in the alignment
    colnames(MSA)<-c(1:nb_pos)
    pos_names <- colnames(MSA) 
	
    gap <- 1-gap_ratio      #gap value indicates the minimal ratio of aa to nb_seq in the MSA 
    if (gap < 1/nb_seq) {
      gap <- 1/nb_seq     # positions must have at leat ONE aa to be taken into account (removes gap column)
    }

    names<-c("A","C","D","E","F","G","H","I","K","L","M","N","P","Q","R","S","T","V","W","Y","-")
    AA<-lapply(1:length(MSA[1,]),function(i){t(table(c(MSA[,i],names),row.names=c(1:(length(MSA[,i])+21))))[1:length(MSA[,i]),-1]})

    names<-c("A","C","D","E","F","G","H","I","K","L","M","N","P","Q","R","S","T","V","W","Y")
    nb_aa <- length(names)
	
    # Determining valid positions with correct gap ratio (equal to 1 - gap argument)
    Valid_pos <- c()
    for(i in 1:nb_pos){
       mat_i <- AA[[i]] #matrix nb_seq*nb_aa
        S_i <- colSums(AA[[i]])
        Tot_i <- sum(S_i)
        if (Tot_i/nb_seq >= gap) {
           Valid_pos <- c(Valid_pos, i)
        }
    }
    nb_Valid_pos <- length(Valid_pos)
	
    MI <- matrix(0, ncol= nb_pos, nrow= nb_pos)
    freq_ij <- matrix(0, ncol= nb_aa, nrow= nb_aa, dimnames=list(names, names))
	
    # Calculating MI score for each valid position
    for(i in 1:nb_Valid_pos){
	pos_i <- Valid_pos[i] #current valid position
	mat_i <- AA[[pos_i]] #matrix nb_seq*nb_aa
	    
	message(paste("pos_i : ", pos_i, "\n"))
	    
	#amino acids frequency at position i in the alignment
	freq_i <- colSums(mat_i)/nb_seq
	    
	for(j in i:nb_Valid_pos){
	    pos_j <- Valid_pos[j]
	    mat_j <- AA[[pos_j]] #matrix nb_seq*nb_aa
		
	    #amino acids frequency at position j in the alignment
	    freq_j <- colSums(mat_j)/nb_seq #matrix 1*nb_aa

	    #amino acids frequency at positions i AND j in the alignment
	    freq_p <- ((t(mat_i))%*%mat_j)/nb_seq #matrix nb_aa*nb_aa
		
	    for (k in names) {
	        for (l in names) {
	    	    freq_ij[k,l]<-freq_i[k]*freq_j[l]
		}
	    }
		
	LOG<-(log(freq_p, base=400)-log(freq_ij, base=400))
	LOG<-replace(LOG, which(LOG=="-Inf"),0)
	LOG<-replace(LOG, which(LOG=="NaN"),0)
		
	MI[pos_i, pos_j]<-sum((freq_p)*(LOG))
	}
    }
	

    COV2<-matrix(0, ncol= nb_pos, nrow= nb_pos)
	
    # MI final value
    COV2 <- MI 
	
    # Setting columns and rows names before matrix reduction
    rownames(COV2)<-pos_names
    colnames(COV2)<-pos_names
	
    COV2 <- COV2+t(COV2) #Complete the second triangular part of the matrix
    diag(COV2) <- diag
	
    # Reduction of the final correlation matrix to the valid positions
    COV2 <- COV2[Valid_pos,]
    COV2 <- COV2[,Valid_pos]
	
    # Removing "Inf" and "NaN" values
    COV2[is.infinite(COV2)] <- 0
    COV2[is.na(COV2)] <- 0
	
    res <- list()
    res$score <- COV2
	
    # Compute and save matrix of Z_scores
    # Mean and stdev must be calculated on off diagonal elements  
    mean_up <- mean(COV2[upper.tri(COV2)])
    stdev <- sd(COV2[upper.tri(COV2)])
    
    COV2 <- (COV2-mean_up)/stdev
    diag(COV2) <- diag
       
    res$Zscore <- COV2
	
    return(res)

}

# Package: Bios2cor 
# This file is part of Bios2cor R package.
# Bios2cor is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# See the GNU General Public License at:
# http://www.gnu.org/licenses/
#
pca_2d <- function(pca_struct, abs= 1, ord= 2, filepathroot=NULL){
#  if(is.null(abs)) abs <- 1
#  if(is.null(ord)) ord <- 2
  
  pca_coord <- pca_struct$coord
  pca_abs <- pca_coord[, abs]
  pca_ord <- pca_coord[, ord]
  
  pca_x <- paste("PCA", abs, sep= "")
  pca_y <- paste("PCA", ord, sep= "")
 
  if(is.null(filepathroot)){
     filename <- paste("PCA_", abs,"_",ord,".png")
  } else {
     filename <- paste(filepathroot,"_PCA_", abs,"_",ord,".png")
  }

  png(filename)
    plot(pca_abs, pca_ord, xlab= pca_x, ylab= pca_y, main= basename(filename))
  dev.off()
}

# Package: Bios2cor 
# This file is part of Bios2cor R package.
# Bios2cor is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# See the GNU General Public License at:
# http://www.gnu.org/licenses/
#
omes <- function(align, gap_ratio = 0.2) {

    if ((gap_ratio < 0) | (gap_ratio > 1)) {
      stop("gap_ratio must be in the [0,1] range.")
    }
 
   diag <- 0	

    msa<-align
    MSA <- matrix(as.vector(unlist(msa)), ncol= length(msa[[1]]), byrow= TRUE)
    nb_pos <- length(MSA[1,]) #number of positions in the alignment
    nb_seq <- length(MSA[,1]) #number of sequences in the alignment
    colnames(MSA)<-c(1:nb_pos)
    pos_names <- colnames(MSA)  
    
    gap <- 1-gap_ratio      #gap value indicates the minimal ratio of aa to nb_seq in the MSA 
    if (gap < 1/nb_seq) {
      gap <- 1/nb_seq     # positions must have at leat ONE aa to be taken into account (removes gap column)
    }


    names<-c("A","C","D","E","F","G","H","I","K","L","M","N","P","Q","R","S","T","V","W","Y","-")

    # Binary matrix indicating which amino acid is present or not at position i in the sequence j
    AA<-lapply(1:nb_pos, function(i){
	    t(table(c(MSA[,i],names),row.names=c(1:(nb_seq+21))))[1:nb_seq, -1]
    })
    
    names<-c("A","C","D","E","F","G","H","I","K","L","M","N","P","Q","R","S","T","V","W","Y")
    nb_aa <- length(names)
    
    COV2<-matrix(0, ncol= nb_pos, nrow= nb_pos)
    
    # Setting columns and rows names before matrix reduction
    rownames(COV2)<-pos_names
    colnames(COV2)<-pos_names 
    
    # Determining valid positions with correct gap ratio (equal to 1 - gap argument)
    Valid_pos <- c()
    for(i in 1:nb_pos){
        mat_i <- AA[[i]] #matrix nb_seq*nb_aa
        S_i <- colSums(AA[[i]])
        Tot_i <- sum(S_i)
        if (Tot_i/nb_seq >= gap) {
           Valid_pos <- c(Valid_pos, i)
        }
    }
    nb_Valid_pos <- length(Valid_pos)
    
    # Calculating score for each valid position
    for(i in 1:nb_Valid_pos){
      pos_i <- Valid_pos[i] #current valid position
      mat_i <- AA[[pos_i]] #matrix nb_seq*nb_aa
      
      message(paste("pos_i : ", pos_i, "\n"))
      
      for(j in i:nb_Valid_pos){
	pos_j <- Valid_pos[j]
	mat_j <- AA[[pos_j]] #matrix nb_seq*nb_aa
	
	#sequences in the alignment without gapped residues at positions i and k (no "-" in the positions i and k)
	Valid<-which(AA[[pos_i]]%*%matrix(1, nrow= nb_aa)*AA[[pos_j]]%*%matrix(1, nrow= nb_aa)!=0)
	
	#the number of sequences in the alignment without gapped residues at positions i and j (no "-" in the positions i and j)
	n<-length(Valid)
	
	Ex<-matrix(as.vector(t(AA[[pos_i]])%*%matrix(1,nrow= nb_seq)), ncol= nb_aa, nrow= nb_aa)*t(matrix(as.vector(t(AA[[pos_j]])%*%matrix(1,nrow= nb_seq)), ncol= nb_aa, nrow= nb_aa))/n
	Obs<-t(AA[[pos_i]])%*%AA[[pos_j]]
	COV2[pos_i, pos_j]<-sum((Obs-Ex)*(Obs-Ex)/n)
      }
    }
    
    #Complete the second triangular part of the matrix
    COV2<-COV2+t(COV2) 
    diag(COV2) <- diag
    
    #Reducting the final correlation matrix to the valid positions
    COV2 <- COV2[Valid_pos,]
    COV2 <- COV2[,Valid_pos]

    #Removing "Inf" values
    COV2[is.infinite(COV2)] <- 0
    COV2[is.na(COV2)] <- 0
    
    res <- list()
  
    # save matrix of score 
    res$score <- COV2
    
    # compute and save matrix of Z_scores
    # mean and stdev must be calculated on off diagonal elements  
    mean_up <- mean(COV2[upper.tri(COV2)])
    stdev <- sd(COV2[upper.tri(COV2)])
    
    COV2 <- (COV2-mean_up)/stdev
    diag(COV2) <- diag
       
    res$Zscore <- COV2
    
    return(res)
}

# Bios2cor is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# See the GNU General Public License at:
# http://www.gnu.org/licenses/
#

delta_filter <- function(entropy, Smin= 0, Smax= 1){
  nb_pos <- length(entropy)
  res_names <- names(entropy)
  
  res <- matrix(1/nb_pos, nrow= nb_pos, ncol= 1)
  
  delta <- unlist(lapply(1:nb_pos, function(pos){ if(entropy[pos] <= Smax & entropy[pos] >= Smin) 1 else 0 }))
  
  res <- delta 
  names(res) <- res_names
  return (res)
}
 

# Bios2cor is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# See the GNU General Public License at:
# http://www.gnu.org/licenses/
#
network.plot <- function(top_pairs, filepathroot){
  pairs_i <- top_pairs$pairs_i
  pairs_j <- top_pairs$pairs_j

   if (is.null(filepathroot)) {
      filename <- paste(tempdir(), "/NETWORK.pdf", sep="")
   } else {
      filename <- paste(filepathroot, "_NETWORK.pdf", sep="")
   }

  pdf(filename)
    # create data:
    links <- data.frame(source= pairs_i, target= pairs_j)
    
    # Turn it into igraph object
    network <- graph_from_data_frame(d= links, directed= F) 
    
    # Count the number of degree for each node:
    deg <- degree(network, mode= "all")
    
    # Plot
    plot(network, vertex.size= deg*6, vertex.color= rgb(0.1,0.7,0.8,0.5))
  
  dev.off()
}

# Bios2cor is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# See the GNU General Public License at:
# http://www.gnu.org/licenses/ 

dynamic_structure <- function(pdb, trj, frames=NULL){

  if (missing(pdb)){
    stop("Missing pdb file")
  }

  if (missing(trj)){
    stop("Missing trajectory file")
  }


  ## Transform the xyz trajectory into dihedral trajectory

  # Reading pdb file
  pdb <- read.pdb(pdb)
  trj <- read.dcd(trj)

  # Selecting only "CA" atoms for superposition
  ca.inds <- atom.select(pdb, elety = "CA")

  # Getting xyz coordinates using fit.xyz from bio3D 
  xyz <- fit.xyz(fixed = pdb$xyz, mobile = trj, fixed.inds = ca.inds$xyz,  mobile.inds = ca.inds$xyz)
  nb_frames <- length(xyz[,1])
  
  # Creating torsion object using the xyz2torsion function from bio3D 
  if(is.null(frames)) {
    tor <- xyz2torsion(pdb, xyz, tbl = "sidechain", ncore= 1)
  } else {
    tor <- xyz2torsion(pdb, xyz[frames,], tbl = "sidechain", ncore= 1)
  }
  nb_torsions <- length(tor[1,])
  nb_frames <- length(tor[,1])
  
  # Associating one letter AA code to each residue, using the pdbseq function from bio3D
  prot.seq <- pdbseq(pdb)
  
  tor.names <- colnames(tor)
  tor.resno <- sub("\\..*$", "", tor.names)
  tor.angle <- sub("^[0-9]*\\.","", tor.names)
  tor.seq <- prot.seq[tor.resno]

  # Filling torsional structure
  res <- list()
  res$pdb <- pdb
  res$trj <- trj
  
  res$ca.inds <- ca.inds
  
  res$xyz <- xyz
  res$nb_frames <- nb_frames
 
  if(!is.null(frames)) {
    res$frames <- frames
  } else {
    res$frames <- c(1:nb_frames)
  }

 
  res$tor <- tor
  res$nb_torsions <- nb_torsions
  
  res$prot.seq <- prot.seq
  
  res$tor.names <- tor.names
  res$tor.resno <- tor.resno
  res$tor.angle <- tor.angle
  res$tor.seq <- tor.seq



  class (res) <- c("structure")  

  return (res)
}

# Bios2cor is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# See the GNU General Public License at:
# http://www.gnu.org/licenses/
#
top_pairs_analysis <- function(corr_matrix, filepathroot= NULL, top= 25, entropy = NULL, filter = NULL ){
  
  if(missing(corr_matrix)){
    stop("A correlation matrix is required")
  }

  if(!is.matrix(corr_matrix)){
     stop("The first argument must be a score or Zscore matrix.")
  }

    if(is.null(filepathroot)){
      filename_contact <- paste("TOP", top, "_CONTACTS.csv", sep="")
      filename_score <- paste("TOP", top, "_SCORES.csv", sep="")
    } else {
      filename_contact <- paste(filepathroot,"_TOP", top, "_CONTACTS.csv", sep="")
      filename_score <- paste(filepathroot,"_TOP", top, "_SCORES.csv", sep="")
    }

 names <- colnames(corr_matrix)
  corr_matrix[lower.tri(corr_matrix)] <- 0
  

  corr_score <- corr_matrix
  names <- colnames(corr_score)
  size_matrix <- length(names)

  # Taking into account the delta filter 
  if(!is.null(filter)){
    for(i in 1:size_matrix){
      for(j in 1:size_matrix){
	name_i <- names[i]
	name_j <- names[j]
	ponderation_i <- filter[name_i]
	ponderation_j <- filter[name_j]
	# score equal to zero when at least one of the two positions is out of the allowed entropy range
	corr_score[i,j] <- corr_score[i,j]*ponderation_i*ponderation_j
      }
    }
  }



  # Extracting top value positions
  x <- which(corr_score >= sort(corr_score, decreasing= TRUE)[top], arr.ind= TRUE)
  
  nb_pairs <- length(x[,1])
  positions <- unique(c(x[,1], x[,2]))
  
  nb_positions <- length(positions)
  
  contacts <- c()
  for(i in 1:nb_positions){

    pos <- positions[i]
    nb_contacts <- 0
    for(j in 1:nb_pairs){
      node_1 <- x[j,1]
      node_2 <- x[j,2]
      if((pos == node_1) | (pos == node_2)){
	nb_contacts <- nb_contacts+1
      }
    }
    contacts[i] <- nb_contacts
  }
  
  # Writing results
  positions <- names[positions]
  
   if (is.null(entropy)){
 # Writing list of top positions with number of contacts 
    head <- paste("position", "contact")
    write(head, filename_contact, append= FALSE)
    for(k in 1:nb_positions){
      position <- positions[k]
      contact <- contacts[k]
      current_line <- paste(position, contact)
      write(current_line, filename_contact, append= TRUE)
    }
 } else {
   entropy <- entropy
   head <- paste("position", "contact", "entropy")
    write(head, filename_contact, append= FALSE)
    for(k in 1:nb_positions){
      position <- positions[k]
      contact <- contacts[k]
      entropy_k <- format(as.numeric(entropy[position]), digits=3, nsmall=3)
      current_line <- paste(position, contact, entropy_k)
      write(current_line, filename_contact, append= TRUE)
    }
}

  
  pairs_i <- x[,1] #pair_i
  pairs_j <- x[,2] #pair_j
  
  res <- list()
  res$pairs_i <- names[pairs_i]
  res$pairs_j <- names[pairs_j]
  res$positions <- positions
  res$contacts <- contacts
  
  ## Writing top score positions
    score <- unlist(lapply(1:nb_pairs, function(a){corr_score[x[a,1], x[a,2]]}))
    index <- sort(score, decreasing= TRUE, index.return= TRUE)$ix
    ordered_x <- x[index,]
    
    write("pair_i pair_j score_ij", filename_score, append= FALSE)
    for(i in 1:nb_pairs){
      pair_i <- names[ordered_x[i,1]]
      pair_j <- names[ordered_x[i,2]]
      score_ij <- score[index[i]]
      score_ij <- format(score_ij,digits=3, nsmall=3)
      score_line <- paste(pair_i, pair_j, score_ij)
      write(score_line, filename_score, append= TRUE)
    }


  
  return (res)
}
 

# Package: Bios2cor 
# This file is part of Bios2cor R package.
# Bios2cor is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# See the GNU General Public License at:
# http://www.gnu.org/licenses/
#
dynamic_entropy <- function(rotamers){

  if(missing(rotamers)){
     stop("A 'rotamers' matrix is required")
  }
 
  if(!is.matrix(rotamers)){
     stop("The argument is not a matrix")
  }

  nb_rotamers <- length(rotamers[1,])
  nb_frames <- length(rotamers[,1])
  max_changes <- nb_frames-1
  dihed_names <- colnames(rotamers)

  entropy <- matrix(0, ncol= nb_rotamers, nrow= 1)

  for(i in 1:nb_rotamers){
      changes <- 0
      for(j in 1:(nb_frames-1)){
	  if(rotamers[j,i] != rotamers[j+1, i]) changes <- changes+1
      }
      entropy[i] <- changes/max_changes
  }
  names(entropy) <- dihed_names
  return (entropy)
}

#  
# Bios2cor is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# See the GNU General Public License at:
# http://www.gnu.org/licenses/
#
write.entropy <- function(entropy, filepathroot){

    if (missing(entropy)) {
      stop("A matrix of type 'entropy' is required.")  
    }

    if (is.null(filepathroot)) {
       filename <- paste(tempdir(),"/ENTROPY.csv",sep = "")
    } else {
       filename <- paste(filepathroot,"_ENTROPY.csv",sep = "")
    }

    nb_pos <- length(entropy)
    positions <- names(entropy)
 
    head <- paste("position", "entropy")
    write(head, file= filename, append= FALSE)
    
    for(i in 1:nb_pos){
      pos <- positions[i]
	if (entropy[i]==''|is.na(entropy[i])){
      		val <- ''
        	current_line <- paste(pos, val)
        	write(current_line, file= filename, append= TRUE)
         } else {
     		val <- as.numeric(entropy[i])
                val <- format(val, digits=3, nsmall=3)
		current_line <- paste(pos, val)
      		write(current_line, file= filename, append= TRUE)
    	}
   }

  entropy <- as.numeric(na.omit(entropy))
    if (is.null(filepathroot)) {
       filename <- "ENTROPY_HIST.png"
    } else {
       filename <- paste(filepathroot,"_ENTROPY_HIST.png",sep = "")
    }

  png(filename)
    hist(entropy)
  dev.off()


}

 

import.msf <- function (file, aa.to.upper = TRUE, gap.to.dash = TRUE) {
  
  if(missing(file)) {
    stop("file is missing")
  }

  # Read as a vector of lines
  lines <- readLines(file)

  # Check msf format
  check1 <- grep("MSF:.*Type:.*Check:.*", lines)
  check2 <- grep("Name:.*Len:.*Check:.*Weight:.*", lines)
  limit <- grep("^//", lines)
  if (length(check1) == 0 || length(check2) == 0 || length(limit) == 0) {
   	stop("file is not in msf format")
	}

  # Get sequence identifiers from header
  id.head <- sub("^\\s*Name:\\s+(\\S+).*$","\\1", lines[check2])
  nb.seq <- length(id.head)

  # Check duplicated identifiers in header
  if(any(duplicated(id.head))){
   stop("duplicated identifiers in header")
  }

  # Get sequence identifiers from alignment
  align <- grep("^\\s*\\S+\\s+[^1-9]+$", lines[limit:length(lines)], value = TRUE)
  id.align <- sub("^\\s*(\\S+)\\s+[^1-9]+$", "\\1", align)

  # Localize sequence pieces for each identifier
  loc <- lapply(seq_len(nb.seq), function(i) {which(id.align == id.head[i])})

  # Paste and clean sequences
  seq <- sapply(loc, function(i) {paste(sub("^\\s*\\S+\\s+([^1-9]+)$", "\\1", align[i]), collapse = "")})
  seq <- gsub("\\s", "", seq)
  
  # Turn aa into upper case
  if (aa.to.upper)
    seq <- toupper(seq)

  # Give a list of split sequences
  seq <- strsplit(seq, split = "")
  names(seq) <- id.head

  # Turn gap into dash character
  if (gap.to.dash)
    seq <- lapply(seq, function (i) {i[is.gap(i)] <- "-"; return(i)})
  class (seq) <- c("align")
  return(seq)
}

# Bios2cor is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# See the GNU General Public License at:
# http://www.gnu.org/licenses/
#
centered_pca <- function(corr_matrix, filepathroot = NULL, filter = NULL, pc= NULL, dec_val= 5){
 
  if(missing(corr_matrix) | is.null(corr_matrix)) {
    stop("A correlation matrix is required")
  }

  if(!is.matrix(corr_matrix)){
     stop("The first argument must be a score or Zscore matrix.")
  }

 
 #Centering the initial matrix
  cor <- corr_matrix
  size <- length(corr_matrix[,1])
  
  #Positions count
  if(is.null(pc)) {
  pc <- size
  print(paste("pc : ", size))
  }

  #identity matrix
  I <- diag(1, size)

  #matrix of ones
  ONES <- matrix(1, nrow = size, ncol = 1)
  
  
  if(is.null(filter)) {
    print("No filter applied")
    #Without filter, the elements have the same mass/weight
    m <- matrix(1/size, nrow= size, ncol= 1)
    mat_names <- colnames(corr_matrix) # names of elements in corr_matrix
  } else {
    print("Filter applied")
    mat_names <- colnames(corr_matrix) #names of elements in corr_matrix
    #filter is limited to the elements present in corr_matrix (may differ in sequence analysis)
    m <- filter[mat_names]    
    SUM <- sum(m)
    m <- m/SUM
  }
  
  #m vector of mass
  BigI<-I-(ONES%*%t(m))
  
  #compute cross-product matrix
  S <-  BigI %*% cor %*% t(BigI)
  
  #Diagonalizing the centered matrix
  res <- list()
  eigen <- eigen(S)
  
  res$eigen <- round(eigen$values[1:pc], 3)
  eigen.perc <- (abs(eigen$values) * 100) / sum(eigen$values[eigen$values>0])
  res$eigen.perc <- round(eigen.perc[1:pc], 3)
  
  all_eigen_values <- eigen$values
  nb_eigenvalues <- length(all_eigen_values)
  
  #only positive eigenvalues are kept
  eigen$vectors <- eigen$vectors[, eigen$values > 0]
  eigen$values <- eigen$values[eigen$values > 0]
  nb_positiv_eigenvalues <- length(eigen$values)
  
  #Storing and plotting eigen values 
    if(is.null(filepathroot)){
      eigen_csv <- "EIGEN.csv"
      eigen_png <- "EIGEN.png"
    } else {
      eigen_csv <- paste(filepathroot, "_EIGEN.csv", sep="")
      eigen_png <- paste(filepathroot, "_EIGEN.png", sep="")
    }
 
    positiv_ev <- sum(eigen$values)
    perc_positiv_ev <- unlist(lapply(eigen$values, function(x){x*100/positiv_ev}))
    
    png(eigen_png)
      plot(1:nb_eigenvalues, all_eigen_values, main= basename(eigen_png))
      abline(h= 0, lty= 2)
    dev.off()

    all_eigen_values <- round(all_eigen_values,digits = 3)
    perc_positiv_ev <- round(perc_positiv_ev, digits= 3)

    csv_tab <- data.frame("eigen$values"= all_eigen_values, "positiv_percent"= c(perc_positiv_ev, rep(0, nb_eigenvalues-nb_positiv_eigenvalues)))
    write.table(csv_tab, row.names= FALSE, file= eigen_csv)
    
 

  res$source<-list()
  res$source$cor <- cor
  res$source$m<-m
  
  #check principal components
  if (pc < 2) pc <- 3
  if (pc > length(eigen$values)) pc <- length(eigen$values)

  #compute the matrix of factor scores
  
  F <- diag(as.vector(m)^(-0.5)) %*% eigen$vectors %*% diag(eigen$values^0.5)
  
  coord <- data.frame(F[, 1:pc])
  rownames(coord) <- rownames(corr_matrix)
  colnames(coord) <- paste ("PC", (1:pc), sep = "")

  res$coord = round(coord, dec_val)
  class (res) <- c("pca")
  return (res)


 
}

# Bios2cor is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# See the GNU General Public License at:
# http://www.gnu.org/licenses/
#
write.scores <- function(correlation, entropy= NULL, filepathroot=NULL){
 
  if (missing(correlation)) {
     stop("A correlation object is required.")
  } 


  if (is.null(filepathroot)) {
     filename <- "CORR_SCORES.csv"
  }else{
     filename <-paste(filepathroot, "_CORR_SCORES.csv", sep="")
  }


  corr_score <- correlation$score
  corr_Zscore <- correlation$Zscore  

  names_c <- colnames(corr_score)
  nb_pos <- length(names_c)
  numbering1 <- grep(".", names_c, fixed= TRUE)
  
  if (!is.null(entropy)){
     names_s  <- names(entropy)
     numbering2 <- grep(".", names_s, fixed= TRUE)
     # Check that both entropy and correlation objects are from a MSA or a trajectory  
     if(((length(numbering1) == 0) && (length(numbering2) != 0)) || ((length(numbering1) != 0) && (length(numbering2) == 0))){
         print("Mismatch in the notation used for the correlation and entropy files. Please verify your files!") 
     }
  }	

 
   if(length(numbering1) != 0){                            
 
     # Read the score_noauto and the Zscore_noauto matrices
      corr_score_noauto <- correlation$score_noauto
      corr_Zscore_noauto <- correlation$Zscore_noauto  

      if (!is.null(entropy)){
         head <- paste("pos_i", "pos_j", "score", "Zscore", "score_noauto", "Zscore_noauto", "entropy_i", "entropy_j")
         write(head, file= filename, append= FALSE)
	  
	 for(i in 1:nb_pos){
            for(j in i:nb_pos){
	       pos_i <- names_c[i]
	       pos_j <- names_c[j]
	       if(pos_i != pos_j){
	  	  entropy_i <- format(as.numeric(entropy[pos_i]), digits=3, nsmall=3)
	  	  entropy_j <- format(as.numeric(entropy[pos_j]), digits=3, nsmall=3)
	  	  score <- format(as.numeric(corr_score[pos_i, pos_j]), digits=3, nsmall=3)
	  	  Zscore <- format(as.numeric(corr_Zscore[pos_i, pos_j]), digits=3, nsmall=3)
	          score_noauto <- format(as.numeric(corr_score_noauto[pos_i, pos_j]), digits=3, nsmall=3)
	          Zscore_noauto <- format(as.numeric(corr_Zscore_noauto[pos_i, pos_j]), digits=3, nsmall=3)
	  
	  	  # Create the line to insert to the file
	  	  current_line <- paste(pos_i, pos_j, score, Zscore, score_noauto, Zscore_noauto, entropy_i, entropy_j) 
	  	  write(current_line, file= filename, append= TRUE)
	       }
            }
         }
      } else {
         head <- paste("pos_i", "pos_j", "score", "Zscore", "score_noauto", "Zscore_noauto")
         write(head, file= filename, append= FALSE)
	  
	 for(i in 1:nb_pos){
            for(j in i:nb_pos){
	       pos_i <- names_c[i]
	       pos_j <- names_c[j]
	       if(pos_i != pos_j){
	  	  score <- format(as.numeric(corr_score[pos_i, pos_j]), digits=3, nsmall=3)
	  	  Zscore <- format(as.numeric(corr_Zscore[pos_i, pos_j]), digits=3, nsmall=3)
	          score_noauto <- format(as.numeric(corr_score_noauto[pos_i, pos_j]), digits=3, nsmall=3)
	          Zscore_noauto <- format(as.numeric(corr_Zscore_noauto[pos_i, pos_j]), digits=3, nsmall=3)
	  
	  	  # Create the line to insert to the file
	  	  current_line <- paste(pos_i, pos_j, score, Zscore, score_noauto, Zscore_noauto) 
	  	  write(current_line, file= filename, append= TRUE)
	       }
            }
         }
      }

   } else {

      if (!is.null(entropy)){
         head <- paste("pos_i", "pos_j", "score", "Zscore", "entropy_i", "entropy_j")
         write(head, file= filename, append= FALSE)

         for(i in 1:nb_pos){
            for(j in i:nb_pos){
	       pos_i <- names_c[i]
	       pos_j <- names_c[j]
	       if(pos_i != pos_j){
	  	  entropy_i <- entropy[pos_i]
	  	  entropy_j <- entropy[pos_j]
	  	  score <- format(as.numeric(corr_score[pos_i, pos_j]), digits=3, nsmall=3)
	   	  Zscore <- format(as.numeric(corr_Zscore[pos_i, pos_j]), digits=3, nsmall=3)
	          if (entropy_i == '') {
	 		entropy_i <- 'ND'
     		  }else{
	  	  entropy_i <- format(as.numeric(entropy_i), digits=3, nsmall=3)
		  }
	          if (entropy_j == '') {
	 		entropy_j <- 'ND'
     		  }else{
	  	  entropy_j <- format(as.numeric(entropy_j), digits=3, nsmall=3)
		  }
	  	  #Creating the line to insert to the file
	  	  current_line <- paste(pos_i, pos_j, score, Zscore, entropy_i, entropy_j)
	  	  write(current_line, file= filename, append= TRUE)
	       }
            }
         }
      }else{
         head <- paste("pos_i", "pos_j", "score", "Zscore")
         write(head, file= filename, append= FALSE)

   	 for(i in 1:nb_pos){
      	    for(j in i:nb_pos){
		pos_i <- names_c[i]
		pos_j <- names_c[j]
		if(pos_i != pos_j){
	  	   score <- format(as.numeric(corr_score[pos_i, pos_j]), digits=3,nsmall=3)
	  	   Zscore <- format(as.numeric(corr_Zscore[pos_i, pos_j]), digits=3,nsmall=3)
	  
	 	   #Creating the line to insert to the file
		   current_line <- paste(pos_i, pos_j, score, Zscore)
		   write(current_line, file= filename, append= TRUE)
	        }
      	     }
    	 }
      }
 
   } 

}

 

# Package: Bios2cor 
# This file is part of Bios2cor R package.
# Bios2cor is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# See the GNU General Public License at:
# http://www.gnu.org/licenses/

dynamic_omes <- function(dynamic_structure, rotamers, res_selection= c("C","I","L","M","V","R","H","K","D","E","N","Q","F","Y","W","T","S","P")){
  
  if(missing(dynamic_structure)){
    stop("An object of class 'structure' is required as first argument")
  }

  if (missing(rotamers)){
    stop("A matrix of type 'rotamers' is required as second argument")
  }

  nb_angles <- length(rotamers[1,])
  nb_frames <- length(rotamers[,1])
  
  pos_names <- colnames(rotamers)
  
  names <- unique(as.vector(rotamers))
  nb_rotamers <- length(names)
  
 # Binary matrix indicating which rotamer is present or not at dihedral position i for each trajectory frame 
  ROT<-lapply(1:nb_angles, function(i){
    t(table(c(rotamers[,i], names), row.names=c(1:(nb_frames+nb_rotamers))))[1:nb_frames,]
  })
  
  COV2 <- matrix(0, ncol= nb_angles, nrow= nb_angles)
  
  #Setting columns and rows names before matrix reduction
  rownames(COV2) <- pos_names
  colnames(COV2) <- pos_names
  
  # Calculating OMES score for each position
  for(i in 1:nb_angles){
    mat_i <- ROT[[i]] #matrix nb_frames*nb_rotamers
    
    cat(paste("pos_i : ", i, "\n"))
    
    for(j in i:nb_angles){
      mat_j <- ROT[[j]] #matrix nb_frames*nb_rotamers   
      Valid <- which(ROT[[i]]%*%matrix(1, nrow= nb_rotamers)*ROT[[j]]%*%matrix(1, nrow= nb_rotamers)!=0)    #possible pairs
      n <- length(Valid)
      Ex <- matrix(as.vector(t(ROT[[i]])%*%matrix(1,nrow= nb_frames)), ncol= nb_rotamers, nrow= nb_rotamers)*t(matrix(as.vector(t(ROT[[j]])%*%matrix(1,nrow= nb_frames)), ncol= nb_rotamers, nrow= nb_rotamers))/n
      Obs <- t(ROT[[i]])%*%ROT[[j]]
      COV2[i, j] <- sum((Obs-Ex)*(Obs-Ex)/n)
    }
  }
  
  COV2<-COV2+t(COV2) #Complete the second triangular part of the matrix
  diag(COV2) <- 0

  #Removing "Inf" and "NA" values
  COV2[is.infinite(COV2)] <- 0
  COV2[is.na(COV2)] <- 0
  
  selected_pos_names <- colnames(COV2)

  if(!is.null(res_selection)){
    #Position selection
    tor.seq <- dynamic_structure$tor.seq
    
    residue_selection.inds <- which(tor.seq %in% res_selection)
    COV2 <- COV2[, residue_selection.inds]
    COV2 <- COV2[residue_selection.inds, ]
    
    nb_angles <- length(COV2[1,])
    selected_pos_names <- colnames(COV2)
  }
  

 
res <- list()
    # save matrix of scores 
    res$score <- COV2
    
    # Compute and save matrix of Z_scores
    # Mean and stdev must be calculated on off diagonal elements  
    mean_up <- mean(COV2[upper.tri(COV2)])
    stdev <- sd(COV2[upper.tri(COV2)])
    COV3 <- (COV2-mean_up)/stdev
    diag(COV3) <- 0
    res$Zscore <- COV3

    # Save matrices of scores and Zscores without auto correlation
    # Create and save matrix with 0 values for autocorrelation (dihedral angles within the same residue)
    COV5 <- COV2
    res_num <- c()
    res_num <- unlist(lapply(selected_pos_names, function(x){strsplit(x, "[.]")[[1]][1]}))

    # Create matrix with NA that will be used for Zscores 
    for(i in 1:nb_angles){
      for(j in i:nb_angles){
	res_i <- res_num[i]
	res_j <- res_num[j]
	
	if(res_i == res_j){
	  COV5[i,j] <- NA
	  COV5[j,i] <- NA
	}
      }
    }
 


    # Save matrix of score with 0 for autocorrelation  
    COV4 <-COV5
    COV4[is.na(COV4)] <- 0
    res$score_noauto <- COV4

    # Compute and save matrix of Z_scores without auto correlation  
    # Mean and stdev are calculated on non NA elements of upper triangle 
    mean_noauto <- mean(COV5[upper.tri(COV5)],na.rm=TRUE)
    stdev_noauto <- sd(COV5[upper.tri(COV5)],na.rm=TRUE)
    
    COV5[is.na(COV5)] <- 0
    COV5 <- (COV5-mean_noauto)/stdev_noauto

    # Correct diag and autocorrelated elements	
    diag(COV5) <- 0
    
    for(i in 1:nb_angles){
      for(j in i:nb_angles){
	res_i <- res_num[i]
	res_j <- res_num[j]
	
	if(res_i == res_j){
	  COV5[i,j] <- 0
	  COV5[j,i] <- 0
	}
      }
    }
  
    res$Zscore_noauto <- COV5

  return(res)
}

# Bios2cor is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# See the GNU General Public License at:
# http://www.gnu.org/licenses/

angle2rotamer <- function(dynamic_structure, conversion_file=system.file("rotamer/dynameomics_rotamers.csv", package= "Bios2cor")){

if (missing(dynamic_structure)){
  stop("A 'dynamic_structure' object is required")
}


  #Importing rotamer information
  angles <- read.table(conversion_file, header= TRUE, fill= TRUE, sep= ",")

  #Importing torsional data from dynamic_structure object
  pdb <- dynamic_structure$pdb
  trj <- dynamic_structure$trj
  
  ca.inds <- dynamic_structure$ca.inds
  
  xyz <- dynamic_structure$xyz
  tor <- dynamic_structure$tor
  nb_torsions <- dynamic_structure$nb_torsions
  nb_frames <- dynamic_structure$nb_frames
  
  prot.seq <- dynamic_structure$prot.seq
  
  tor.names <- dynamic_structure$tor.names
  tor.resno <- dynamic_structure$tor.resno
  tor.angle <- dynamic_structure$tor.angle
  tor.seq <- dynamic_structure$tor.seq

  converted_angles <- matrix("", ncol= nb_torsions, nrow= nb_frames)
  colnames(converted_angles) <- tor.names
  
  #converting dihedral angles into rotamers 
  for(i in 1:nb_frames){
       for(j in 1:nb_torsions){
	  residue <- tor.seq[j]
	  dihedral <- tor.angle[j]
	  angle <- tor[i,j]
	  
	  #select the rotamers possible for the dihedral angle and residue under consideration
	  selected_torsion <- as.matrix(angles[(angles[,1] == residue) & (angles[,2] == dihedral),])
	  rotamer_nb <- length(selected_torsion[,1])
	  
	  #check if the selected torsion is contained in the conversion file
	  if(rotamer_nb != 0){
	    for(r in 1:rotamer_nb){
		ang_1 <- selected_torsion[r,4]
		ang_2 <- selected_torsion[r,5]
		if(ang_1 == "0-" | ang_1 == "0+") ang_1 <- "0"
		if(ang_2 == "0-" | ang_2 == "0+") ang_2 <- "0"
		
		min_angle <- as.numeric(ang_1)
		max_angle <- as.numeric(ang_2)
		
		special_interval <- FALSE #the interval is greater than 180 degrees
		if(min_angle > max_angle) special_interval <- TRUE
		
		if(special_interval){
		    if((min_angle <= angle & angle <= 180) | (-180 <= angle & angle <= max_angle)) converted_angles[i,j] <- selected_torsion[r,3]
		} else {
		    if(min_angle <= angle & angle <= max_angle) converted_angles[i,j] <- selected_torsion[r,3]
		}
	    }
	  } else {
	      warning(paste("dihedral angle not found in the conversion file \n"))
	      message(paste("residue = ", residue, "\n"))
	      message(paste("dihedral = ", dihedral, "\n"))
	  }
      }
  }
  
  return (converted_angles)
}

"xyz2torsion" <-
function(pdb, xyz, tbl = c("basic", "mainchain", "sidechain", 
    "all", "phi", "psi", paste("chi", 1:5, sep="")), ncore = NULL) {

  if(length(pdb$xyz) != ncol(xyz)) 
     stop("Number of atoms in PDB doesn't match xyz")
  ncore <- setup.ncore(ncore, bigmem = TRUE)
  
  tor.names <- c("phi", "psi", paste("chi", 1:5, sep=""))
  tbl <- match.arg(tbl)
  tbl <- switch(tbl, 
     "basic" = tor.names[1:3],
     "mainchain" = tor.names[1:2], 
     "sidechain" = tor.names[3:7],
     "all" = tor.names,
     tbl
  )
  tor <- torsion.pdb(pdb)
  fill <- !is.na(t(tor$tbl[, tbl]))
  resno <- rep(pdb$atom[pdb$calpha, "resno"], each = length(tbl))
  cname <- paste(resno[fill], rep(tbl, ncol(fill))[fill], sep=".")
  n <- sum(fill)

  pb <- txtProgressBar(min=0, max=nrow(xyz), style=3)
  if(ncore > 1) {
     tor <- big.matrix(nrow(xyz), n, init=NA, type="double")
     iipb <- big.matrix(1, nrow(xyz), init=NA)
     mclapply(1:nrow(xyz), function(i) {
        pdb$xyz <- xyz[i, ]
        tor[i, ] <- t(torsion.pdb(pdb)$tbl[, tbl])[fill]
        iipb[1, i] <- 1
        setTxtProgressBar(pb, sum(!is.na(iipb[1,])))
        return()
     } )
     tor <- tor[,]; gc()
  } else {
     tor <- t( sapply(1:nrow(xyz), function(i) {
        pdb$xyz <- xyz[i, ]
        tor <- t(torsion.pdb(pdb)$tbl[, tbl])[fill]
        setTxtProgressBar(pb, i)
        return(tor)
     } ) )
  }
  close(pb)
  colnames(tor) <- cname
  return(tor)
} 

# Package: Bios2cor 
# This file is part of Bios2cor R package.
# Bios2cor is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# See the GNU General Public License at:
# http://www.gnu.org/licenses/

dynamic_circular <- function(dynamic_structure, res_selection= c("C","I","L","M","V","R","H","K","D","E","N","Q","F","Y","W","T","S","P")){

  if(missing(dynamic_structure)){
    stop("A 'dynamic_structure' object is required")
  }

  #  Importing torsional information
  tor <- dynamic_structure$tor
  nb_torsions <- length(tor[1,])
  
  # Torsional angles
  sidechain.tor <- tor[, 1:nb_torsions]
  
  dihed_names <- colnames(sidechain.tor)
  
  # Transforms the torsion matrix in circular object (circular package)
  dihed <- circular(sidechain.tor, units="degrees", type="angles", modulo="2pi")

  # Computes circular correlation
  dihed_corr <- cor.circular(dihed)

  colnames(dihed_corr) <- dihed_names
  rownames(dihed_corr) <- dihed_names
  
  diag(dihed_corr) <- 0
  
  # Squares because only absolute value of correlation matters
  dihed_corr <- dihed_corr^2    
  
  dihed_corr_names <- colnames(dihed_corr)
  
  if(!is.null(res_selection)){
    # Position selection
    tor.seq <- dynamic_structure$tor.seq
    
    residue_selection.inds <- which(tor.seq %in% res_selection)
    dihed_corr <- dihed_corr[, residue_selection.inds]
    dihed_corr <- dihed_corr[residue_selection.inds, ]
    
    nb_angles  <- length(dihed_corr[1,])
    selected_dihed_names <- colnames(dihed_corr)
  }
  

  COV2 <- dihed_corr

  # Removing "Inf" and "NA" values
  COV2[is.infinite(COV2)] <- 0
  COV2[is.na(COV2)] <- 0


res <- list()
    # Save matrix of scores 
    res$score <- COV2
    
    # Compute and save matrix of Z_scores
    # Mean and stdev must be calculated on off diagonal elements  
    mean_up <- mean(COV2[upper.tri(COV2)])
    stdev <- sd(COV2[upper.tri(COV2)])
    COV3 <- (COV2-mean_up)/stdev
    diag(COV3) <- 0
    res$Zscore <- COV3

    # Save matrices of score and Zscores without auto correlation
    # Create and save matrix with 0 values for autocorrelation (dihedral angles within the same residue)
    COV5 <- COV2
    res_num <- c()
    res_num <- unlist(lapply(selected_dihed_names, function(x){strsplit(x, "[.]")[[1]][1]}))

    # Create matrix with NA that will be used for Zscores 
    for(i in 1:nb_angles){
      for(j in i:nb_angles){
	res_i <- res_num[i]
	res_j <- res_num[j]
	
	if(res_i == res_j){
	  COV5[i,j] <- NA
	  COV5[j,i] <- NA
	}
      }
    }
 
    # Save matrix of score with 0 for autocorrelation  
    COV4 <-COV5
    COV4[is.na(COV4)] <- 0
    res$score_noauto <- COV4

    # Compute and save matrix of Z_scores without auto correlation  
    # Mean and stdev are calculated on non NA elements of upper triangle 
    mean_noauto <- mean(COV5[upper.tri(COV5)],na.rm=TRUE)
    stdev_noauto <- sd(COV5[upper.tri(COV5)],na.rm=TRUE)
    
    COV5[is.na(COV5)] <- 0
    COV5 <- (COV5-mean_noauto)/stdev_noauto

    ### Correct diag and autocorrelated elements	
    diag(COV5) <- 0
    
    for(i in 1:nb_angles){
      for(j in i:nb_angles){
	res_i <- res_num[i]
	res_j <- res_num[j]
	
	if(res_i == res_j){
	  COV5[i,j] <- 0
	  COV5[j,i] <- 0
	}
      }
    }
  
    res$Zscore_noauto <- COV5

return(res)
}

# Bios2cor is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# See the GNU General Public License at:
# http://www.gnu.org/licenses/
#
angles.plot <- function(dynamic_structure, filepathroot, angles=NULL){
 
if (missing(dynamic_structure))
  stop("A 'dynamic_structure' object is required")

if (is.null(angles)){
  angles <- dynamic_structure$tor.names
  }


if (is.null(filepathroot)) {
  final_filepath <- paste(tempdir(), "/ANGLES.pdf",sep="")
  final_csv <- paste(tempdir(), "/ANGLES.csv",sep="")
} else {
  final_filepath <- paste(filepathroot, "_ANGLES.pdf",sep="")
  final_csv <- paste(filepathroot, "_ANGLES.csv",sep="")
}


  #Importing torsional and angles data
  tor <- dynamic_structure$tor 
  nb_frames <- length(tor[,1])
  nb_angles <- length(angles)
  frames <- dynamic_structure$frames

  #Generating ordered plots
  nb_max_figures <- 35
  nb_col <- 5
  
    pdf(final_filepath)

	#save user's parameters to reset them on exit
	oldpar <- par(no.readonly = TRUE)
	on.exit(par(oldpar)) 

	layout(matrix(1:nb_max_figures, ncol= nb_col, byrow= TRUE))     # plot up to 7 rows of 5 figures / page
    
    #Setting the margins of the graph in inches (mai)  
    par(mai= c(0.25, 0.25, 0.2, 0.2))
    
    #Alphanumerical graph ordering 
    graph_order <- c(1:nb_angles)
    graph_order <- sort(unlist(as.integer(lapply(strsplit(angles, "\\."), "[[", 1))), index.return= TRUE)$ix
    
    for(i in graph_order){
      angle_name <- angles[i]
      dihedral_angles <- tor[, angle_name]
      plot(x= 1:nb_frames, y= dihedral_angles, cex= 0.1, xlab= "frames", ylab="dihedral angles", main= angle_name)
    }
    
  dev.off()


 tor <- tor[,angles]    
 write.csv(cbind(frames,round(tor, digits=2)), file = final_csv)



}

# Bios2cor is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# See the GNU General Public License at:
# http://www.gnu.org/licenses/
#
scores.boxplot <- function(corr_matrix_list, name_list, filepathroot, elite=25, high=275){
  if (is.null(filepathroot)) {
    filename <- paste(tempdir(), "/BOXPLOT.png", sep = "")
  }else{
    filename <- paste(filepathroot, "_BOXPLOT.png", sep = "")
  }
  corr_tab <- sapply(corr_matrix_list, function(x){as.vector(x)})
  nb_objects <- length(corr_matrix_list)
  

  message(paste("Boxplot elements :", nb_objects))
  
  png(filename, width = 600, height = 400, units = "px", pointsize = 12)
    oldpar <- par(no.readonly = TRUE) 
	on.exit(par(oldpar)) 
	boxplot(corr_tab, names = name_list, xlab="Z-score", cex.lab=1.5, cex.axis=0.90, col = "grey", outcol = "grey", horizontal=T, las=1)
    for(i in 1:nb_objects){
      matrix <- corr_matrix_list[i]
      X <- upper.tri(matrix,diag=FALSE)             
      increasing_corr <- sort(corr_tab[,i])
      decreasing_corr <- sort(corr_tab[,i], decreasing= TRUE)
      
      top_high <- decreasing_corr[1:high*2]
      points(top_high, rep(i,length(top_high)), pch=16, col="dodgerblue")
      
      top_elite <- decreasing_corr[1:elite*2]
      points(top_elite, rep(i,length(top_elite)), pch=16, col="blue")
      
      bottom_high <- increasing_corr[1:high*2]
      points(bottom_high, rep(i,length(bottom_high)), pch=16, col="pink")
      
      bottom_elite <- increasing_corr[1:elite*2]
      points(bottom_elite, rep(i,length(bottom_elite)), pch=16, col="red")
    }
    
    #Draw a line at position v
    abline(v=4, lty=2)			
    
    #Box with special line width
    box(lwd = 3)
  dev.off()
}

\name{top_pairs_analysis}

\Rdversion{1.1}

\alias{top_pairs_analysis}

\title{
  Calculates the number of contacts for each element of the top pairs 
}

\description{
  Given a correlation object, calculates the number of pairs (contacts) each element in the top X pairs are involved in
}

\usage{
  top_pairs_analysis(corr_matrix, filepathroot=NULL, top=25, entropy=NULL, filter=NULL)
}

\arguments{
 \item{corr_matrix}{
  One of the matrices created by a correlation/covariation function (\code{\link{omes}}, \code{\link{mip}}, \code{\link{elsc}}, \code{\link{mcbasc}}, \code{\link{dynamic_circular}}, \code{\link{dynamic_omes}}, \code{\link{dynamic_mip}}).
 } 
 \item{filepathroot}{
  The root of the full path names of the output files for top_pairs_analysis. Default is NULL (two csv files are created : TOPn_CONTACTS.csv and TOPn_SCORES.csv, where n is the number of top pairs). If not NULL, extentions "-TOPn_CONTACTS.csv" and "_TOPn_SCORES.csv" are added to the root name.
 }
 \item{top}{
  A integer indicating the number of top pairs used for this analysis. Default is 25.
 } 
 \item{entropy}{
  An object created by the entropy or dynamic_entropy function. Default is NULL.
 } 
 \item{filter}{
   A vector created by the \code{\link{delta_filter}} function to limit the analysis to elements in the given entropy/dynamic_entropy range. DEfault is NULL.
 } 
}

\details{
  This function sorts element pairs by correlation/covariation scores and analyzes the top X pairs to determine the number of pairs (contacts) each element of the top X pairs is involved in. If filter is TRUE, only the scores of elements in the delta filter defined entropy range are taken into account. Results are written as .csv files.
}

\value{
returns an object of class 'top_pairs' which is a named list of four elements for subsequent network representation with the \code{network.plot} function:
  \item{pair_i}{
  a vector containing the name of element i in the ordered top pairs
  }
  \item{pair_j}{
  a vector containing the name of element j in the ordered top pairs
  }
  \item{positions}{
  a vector containing the positions in the top n pairs
  }
  \item{contacts}{
  a vector containing the number of contacts of the positions in the top n pairs
  }
returns also two .csv files containing scores and contacts of the top n pairs for subsequent network representation with Cytoscape.

}

\author{
  Antoine GARNIER and Marie CHABBERT
}


\examples{
  #File path for output files
  wd <- tempdir()
  #wd <-getwd() 
  file <- file.path(wd,"test_seq") 

  #Importing MSA file
  msf <- system.file("msa/toy_align.msf", package = "Bios2cor")
  align <- import.msf(msf)

  #Creating entropy object
  entropy <- entropy(align)

  #Creating OMES correlation object
  omes <- omes(align, gap_ratio = 0.2)

  #Selecting correlation matrix
  omes <-omes$Zscore
   
  #Creating top_pairs object and writing scores and contacts to csv files  
  top_pairs <- top_pairs_analysis(omes, file, top = 25, entropy=entropy)
}

\name{xyz2torsion}
\alias{xyz2torsion}
\title{ Convert Cartesian Coordinates to Torsion Angles }
\description{
  Convert cartesian coordinate matrix to torsion angles with 
  function \code{\link{torsion.pdb}}.
}
\usage{
xyz2torsion(pdb, xyz, tbl = c("basic", "mainchain", 
     "sidechain", "all", "phi", "psi", paste("chi", 1:5, sep="")), ncore = NULL)
}
\arguments{
  \item{pdb}{ A PDB structure object as obtained from \code{\link{read.pdb}}. }
  \item{xyz}{ Cartesian coordinates as a Mx(3N) matrix. }
  \item{tbl}{ Names of torsion angles to calculate. }
  \item{ncore}{ Number of CPU cores used to do the calculation. By default (NULL), use
    all detected CPU cores. }
}
\details{
  Available values for the argument \sQuote{tbl} include:
  \itemize{
     \item Basic: "phi", "psi", "chi1".
     \item Mainchain: "phi", "psi".
     \item Sidechain: "chi1", "chi2", "chi3", "chi4", "chi5".
     \item All: all of the above.
     \item Each individual angle.
  }
}
\value{
  Returns a MxP matrix, where M is the number of frames and P the number of valid torsion angles.
}
\references{
  Grant, B.J. et al. (2006) \emph{Bioinformatics} \bold{22}, 2695--2696.
}
\author{ Xin-Qiu Yao }
\seealso{ \code{\link{torsion.xyz}}, \code{\link{torsion.pdb}}
 }
\note{
 New function from the bio3d package, available at <https://github.com/Grantlab/bio3d/blob/master/new_funs/xyz2torsion.R>
}
\examples{
  pdb <- system.file("rotamer/toy_coordinates.pdb", package= "Bios2cor")
  trj <- system.file("rotamer/toy_dynamics.dcd", package= "Bios2cor")
  
  pdb <- read.pdb(pdb)
  trj <- read.dcd(trj)
 
  #Selecting only "CA" atoms
  ca.inds <- atom.select(pdb, elety = "CA")

  #Getting xyz coordinates using fit.xyz form bio3d package
  xyz <- fit.xyz(fixed = pdb$xyz, mobile = trj,fixed.inds=ca.inds$xyz,mobile.inds=ca.inds$xyz)
 
  frames <- seq(from= 1, to= 40, by= 2)
  
  #Creating torsion object for side chains using xyz2torsion function from bio3d package
  tor <- xyz2torsion(pdb, xyz[frames,], tbl = "sidechain", ncore= 1)
}

\name{pca_2d}

\Rdversion{1.1}

\alias{pca_2d}

\title{
  PCA projection on two dimensions
}

\description{
  Given an object of class 'pca'  (result of the \code{\link{centered_pca}} function), draws a graph of the projection of the elements on two dimensions (first and second components by default)
}

\usage{
  pca_2d(pca_struct, abs= 1, ord= 2, filepathroot=NULL)
}

\arguments{
 \item{pca_struct}{
  An object created by the \code{\link{centered_pca}} function
 }
 \item{abs}{
  An integer which corresponds to the x axis of the projection plane. Default is 1 (first component)
 }
 \item{ord}{
  An integer which corresponds to the y axis of the projection plane. Default is 2 (second component)
 }
 \item{filepathroot}{
  The root of the full path name for the output file. Default is NULL (a "PCA_abs_ord.png" file will be created). If not NULL, the filepathroot will have the "_PCA_abs_ord.png" extension. 
 }
}

\value{
  A 2D graph of the projection of the elements on selected two principal components
}

\author{
  Antoine GARNIER
}

\examples{
  #File path for output files
  wd <- tempdir()
  #wd <-getwd() 
  file <- file.path(wd,"test_seq2") 

  #Importing MSA file
  msf <- system.file("msa/toy_align.msf", package = "Bios2cor")
  align <- import.msf(msf)

  #Creating OMES correlation object
  omes <- omes(align, gap_ratio = 0.2)

  #Creating entropy object
  #entropy <- entropy(align)

  #Selecting correlation matrix
  omes <-omes$Zscore

  #Creating PCA object for selected correlation matrix and saving eigen values
  pca <- centered_pca(omes, filepathroot= file, pc= NULL, dec_val= 5, filter= NULL)

  #Creating 2D plot of elements projected on selected [abs,ord] plane 
  pca_2d(pca, abs = 1, ord = 3, file)

  #Creating 2D plot of elements projected on default [1,2] plane 
  #pca_2d(pca)  
}

\name{import.msf}
\Rdversion{1.1}
\alias{import.msf}
\title{
Reads a multiple sequence alignment file in MSF format
}
\description{
Reads a Multiple Sequence Alignment (MSA) file in MSF format (.msf extension).
}
\usage{
import.msf(file, aa.to.upper = TRUE, gap.to.dash = TRUE)
}
\arguments{
  \item{file}{
  a string of characters to indicate the name of the MSA file to be read.
}
  \item{aa.to.upper}{
  a logical value indicating whether amino acids should be converted to upper case (TRUE) or not (FALSE). Default is TRUE.
}
  \item{gap.to.dash}{
  a logical value indicating whether the dot (.) and tilde (\eqn{\sim}) gap symbols should be converted
  to the dash (-) character (TRUE) or not (FALSE). Default is TRUE.
}
}
\details{
Initially, Multiple Sequence Format (MSF) was the multiple sequence alignment format of
the Wisconsin Package (WP) or GCG (Genetic Computer Group). This package is a suite of over 130
sequence analysis programs for database searching, secondary structure prediction
or sequence alignment. Presently, numerous multiple sequence alignment editors
(Jalview and GeneDoc for example) can read and write MSF files.
  
MSF file displays several specificities:
\itemize{
\item a header containing sequence identifiers and characteristics (length, check and weight).
\item a separator symbolized by 2 slashes (//).
\item sequences of identifiers, displayed by consecutive blocks.
}
}
\value{
A object of class 'align', which is a named list whose elements correspond to sequences, in the form of character vectors.
}
\note{
The \code{import.msf} function was developed for the \code{bios2mds} R package (Julien PELE [aut], Jean-Michel BECU [aut], Marie CHABBERT [cre]).

It checks the presence of duplicated identifiers in header. Sequences whose identifiers are missing in header are ignored.
}
\author{
Julien PELE
}
\seealso{
\code{read.alignment} function from \code{seqinr} package.\cr
\code{read.GDoc} function from \code{aaMI} package (archived).
}
\examples{
  #Importing MSA file in MSF format
  aln <- import.msf(system.file("msa/toy_align.msf", package = "Bios2cor"))
}

\name{mcbasc}

\Rdversion{1.1}

\alias{mcbasc}

\title{
  McBASC (McLachlan Based Substitution Correlation) function
}

\description{
  Calculates a score for each pair of residus in the sequenec alignment. 
  It relies on a substitution matrix giving a similarity score for each pair of amino acids.
}

\usage{
  mcbasc(align, gap_ratio= 0.2)
}

\arguments{
 \item{align}{
 An object of class 'align' created by the \code{\link{import.msf}} or the \code{\link{import.fasta}} function from a sequence alignment 
 }
 \item{gap_ratio}{
  Numeric value between 0 and 1 indicating the maximal gap ratio at a given position in the MSA for this position to be taken into account. Default is 0.2, positions with more than 20 percent of gaps will not be taken into account in the analysis. When gap_ratio is 1 or close to 1, only positions with at least 1 aa are taken into account (positions with only gaps are excluded). 
 }
}

\details{
  The McBASC score at position [i,j] has been computed with a formula which was initially proposed by Valencia and coworkers(1) as follow :
  
  \deqn{{McBASC(i,j)} = \frac{1}{N^2\sigma(i)\sigma(j)} \sum_{k,l}^{ } (SC_{k,l}(i)-SC(i))(SC_{k,l}(j)-SC(j))}
  
  where :
    \itemize{
      \item {\emph{\eqn{SC_{k,l}(i)}}} {is the score for the amino acid pair present in sequences k and l at position i}
      \item {\emph{\eqn{SC_{k,l}(j)}}} {is the score for the amino acid pair present in sequences k and l at position j}
      \item {\emph{\eqn{SC(i)}}} {is the average of all the scores \emph{\eqn{SC_{k,l}(i)}}}
      \item {\emph{\eqn{SC(j)}}} {is the average of all the scores \emph{\eqn{SC_{k,l}(j)}}}
      \item {\emph{\eqn{\sigma(i)}}} {is the standard deviation of all the scores \emph{\eqn{SC_{k,l}(i)}}}
      \item {\emph{\eqn{\sigma(j)}}} {is the standard deviation of all the scores \emph{\eqn{SC_{k,l}(j)}}}
    }
}

\value{
  A list of two elements which are numeric matrices containing the mcbasc scores and Z-scores for each pair of elements.
}

\author{
  Madeline DENIAUD and Marie CHABBERT
}

\references{
 (1) Gobel U, Sander C, Schneider R, Valencia A. Correlated mutations and residue contacts in proteins. Proteins 1994;18:309-317.
}


\examples{
  #Importing MSA file
  align <- import.msf(system.file("msa/toy_align.msf", package = "Bios2cor"))

  #Creating correlation object with McBASC method for positions with gap_ratio < 0.2 (Default)
  mcbasc <- mcbasc(align, gap_ratio = 0.2)
}

\name{scores.boxplot}

\Rdversion{1.1}

\alias{scores.boxplot}

\title{
  Creates boxplots of correlation/covariation scores 
}

\description{
  Given a list of correlation/covariation matrices, build boxplots for comparative purposes.
}

\usage{
  scores.boxplot(corr_matrix_list, name_list, filepathroot=NULL, elite=25, high=275)
}

\arguments{
 \item{corr_matrix_list}{
  A list of correlation/covariation matrices to be compared
 }
 \item{name_list}{
  The names of the correlation/covariation matrices 
 }
 \item{filepathroot}{
  The root of the full path name for the output file. Default is NULL (a BOXPLOT.png file will be created). If not NULL, the "_BOXPLOT.png" extension is added to the filepathroot. 
 }
 \item{elite}{
  An integer to determine the number of pairs with the highest and lowest scores (e.g. 25: pairs ranked 1 to 25 in decreasing or increasing order) to be colored with the "elite" color codes. Default is 25.
 }
 \item{high}{
  An integer to determine the number of pairs with the next highest and lowest scores (e.g. 275: pairs ranked 26 to 275 in decreasing or increasing order) to be colored with the "high" color codes. Default is 275.
 }
}

\details{
  The correlation/covariation matrices contain the correlation/covariation scores for each pair of elements [i,j].
  The boxplots will allow comparing these scores using color codes : the highest values are dark blue, the next highest values are light blue, the lowest values are red and the next lowest values are pink.
}

\value{
  A pdf figure with boxplots to compare correlation/covariation scores
}

\references{
For an application of these boxplots, see : 

Pele J, Abdi H, Moreau M, Thybert D and Chabbert M (2011) Multidimensional scaling reveals the main evolutionary pathways of class A G-protein-coupled receptors. \emph{PLoS ONE} \bold{6}: e19094. doi:10.1371.
}


\author{
  Julien PELE and Antoine GARNIER
}

\examples{
  #File path for output file
  wd <- tempdir()
  #wd <-getwd() 
  file <- file.path(wd,"test_seq") 

  #Importing MSA file
  msf <- system.file("msa/toy_align.msf", package = "Bios2cor")
  align <- import.msf(msf)

  #Creating OMES correlation object
  omes <- omes(align, gap_ratio = 0.2)

  #Creating MIP correlation object
  #mip <- mip(align, gap_ratio = 0.2)

  #Selecting correlation matrices
  omes <-omes$Zscore
  #mip <- mip$Zscore

  #Creating a list of matrices and plotting the boxplots in a graph
  #Two matrices
  #corr_matrix_list <- list(omes,mip)
  #name <- c("omes","mip")
  #One matrix
  corr_matrix_list <- list(omes)
  name <- c("omes")
  scores.boxplot(corr_matrix_list, name, file, 25, 275)
}

\name{dynamic_omes}

\Rdversion{1.1}

\alias{dynamic_omes}

\title{
  OMES (Observed minus Expected Squared) function applied to rotamers in molecular dynamics simulations
}

\description{
  Calculates difference between observed and expected occurrences of each possible pair of rotamers (x, y) for i and j dihedral angles over all frames
}

\usage{
  dynamic_omes(
    dynamic_structure,
    rotamers,
    res_selection= 
      c("C","I","L","M","V","R","H","K","D","E","N","Q","F","Y","W","T","S","P")
  )
}

\arguments{
 \item{dynamic_structure}{
   An object of class 'structure' that is created by the \code{\link{dynamic_structure}} function
 }
 \item{rotamers}{
 A character matrix that is produced by the \link{angle2rotamer} function. The matrix indicates the rotameric state of each side chain dihedral angle for each frame of the trajectory.
 }
 \item{res_selection}{
  List of amino acids that will be taken into account in the correlation/covariation matrix. By default, all the amino acids are taken into account except Gly and Ala, with no side chain dihedral angles.  
 }
}

\details{
  The OMES score for angles [i,j] has been computed with the following formula :
  
  \deqn{{OMES(i,j)} = \frac{1}{N} \sum_{x,y}^{ }(N_{x,y}^{obs}(i,j)-N_{x,y}^{ex}(i,j))^2}
  
  with : \emph{\eqn{N_{x,y}^{ex}(i,j) = p_{x}(i)p_{y}(j)N}}

  where :
    \itemize{
      \item {\emph{\eqn{N_{x,y}^{obs}(i,j)}}}{ is number of times that each (x,y) rotamer pair is observed at angles i and j}
      \item {\emph{\eqn{N_{x,y}^{ex}(i,j)}}}{ is number of times that each (x,y) rotamer pair would be expected, based on the frequency of rotamer x and y at angles i and j}
      \item {\emph{\eqn{N}}}{ is the number of frames}
      \item {\emph{\eqn{p_{x}(i)}}}{ is the frequency of rotamer x at angle i}
      \item {\emph{\eqn{p_{y}(j)}}}{ is the frequency of rotamer y at angle j}
    }
}

\value{
  returns a list of four elements which are numeric matrices containing (1) the correlation/covariation scores for each pair of rotamers (score), (2) the Z-scores for each pair of rotamers (Zscore), (3) the correlation/covariation scores for each pair of rotamers with zero values for autocorrelation (correlation within the same side chain) (score_noauto) and (4) the Z-scores calculated without autocorrelation pairs and zero values for autocorrelation pairs (Zscore_noauto). 
}

\author{
   Antoine GARNIER, Lea BELLENGER, and Marie CHABBERT 
}

\references{
Fodor AA and Aldrich RW. Influence of conservation on calculations of amino acid covariance in multiple sequence alignments. Proteins. 2004;56(2):211-21.
}

\examples{
  #Reading pdb and dcd files
  pdb <- system.file("rotamer/toy_coordinates.pdb", package= "Bios2cor")
  trj <- system.file("rotamer/toy_dynamics.dcd", package= "Bios2cor")

  #Creating dynamic_structure object
  wanted_frames <- seq(from = 5, to = 40, by = 10)
  dynamic_structure <- dynamic_structure(pdb, trj, wanted_frames)

  #Creating rotamers object using conversion_file
  conversion_file <- system.file("rotamer/dynameomics_rotamers.csv", package= "Bios2cor")
  rotamers <- angle2rotamer(dynamic_structure, conversion_file)

  #Creating correlation object for selected residues with OMES method
  wanted_residues <- c("W")
  omes_corr <- dynamic_omes(dynamic_structure, rotamers, wanted_residues)
}

\name{write.scores}

\Rdversion{1.1}

\alias{write.scores}

\title{
  Creates a correlation/covariation ouput file 
}

\description{
  Given a correlation object, writes the score, the Zscore and, optionally the entropy, for each pair of elements in a csv file.
}

\usage{
  write.scores(correlation, filepathroot, entropy= NULL)
}

\arguments{
 \item{correlation}{
  An object created by a correlation/covariation function (\code{\link{omes}}, \code{\link{mi}}, \code{\link{mip}}, \code{\link{elsc}}, \code{\link{mcbasc}}, \code{\link{dynamic_circular}}, \code{\link{dynamic_omes}}, \code{\link{dynamic_mi}}, \code{\link{dynamic_mip}}
 }
 \item{entropy}{
  An object created by the \code{\link{entropy}} function 
 }
 \item{filepathroot}{
  The root of the full path name for the output file. DEfault is NULL, a "CORR_SCORES.csv" file is created in tempdir(). If not NULL, the "_CORR_SCORES.csv" extension is added to the filepathroot.
 }
}

\details{
  Elements represent positions in multiple sequence alignments and side chain dihedral angles in molecular dynamic simulations (MD). 

  In sequence analysis, the correlation object contains two matrices with the correlation/covariation scores and Z-scores for each pair of elements [i,j].   
  If entropy = NULL, each line of the output file will contain element  i, element  j, score[i,j], and Z-score[i,j]. 
  If entropy is not NULL, each line of the output file will contain element  i, element  j, score[i,j], Zscore[i,j], entropy[i], and entropy[j].  

  In MD analysis, the correlation object contains four matrices with (1) the correlation/covariation scores for each pair of rotamers (score), (2) the Z-scores for each pair of rotamers (Zscore), (3) the correlation/covariation scores for each pair of rotamers with zero values for autocorrelation (correlation within the same side chain) (score_noauto) and (4) the Z-scores calculated without autocorrelation pairs and zero values for autocorrelation pairs (Zscore_noauto). 
  If entropy = NULL, each line of the output file will contain element  i, element  j, score[i,j], Zscore[i,j], score_noauto[i,j], and Zscore_noauto[i,j].
  If entropy is not NULL, each line of the output file will contain element  i, element  j, score[i,j], Zscore[i,j], score_noauto[i,j], Zscore_noauto[i,j], entropy[i], and entropy[j].

}


\value{
  A csv file containing the correlation/covariation scores and optionally the entropies.
}

\author{
  Antoine GARNIER and Marie CHABBERT
}

\examples{

#Example for MSA 
  #File path for output files
  out <- tempdir()
  file <- file.path(out,"test_seq") 

  #Importing MSA file
  msf <- system.file("msa/toy_align.msf", package = "Bios2cor")
  align <- import.msf(msf)

  #Creating correlation and entropy objects
  correlation <- omes(align, gap_ratio= 0.2)
  entropy <- entropy(align)
  
  #Writing results to csv file
  write.scores(correlation, file, entropy )

###Example for MD
  #File path for output files
  wd <- tempdir()
  #wd <-getwd() 
  file <- file.path(wd,"test_dyn") 

  #Reading the pdb and dcd files and the angles to rotamers conversion file
  pdb <- system.file("rotamer/toy_coordinates.pdb", package= "Bios2cor")
  trj <- system.file("rotamer/toy_dynamics.dcd", package= "Bios2cor")
  conversion_file <- system.file("rotamer/dynameomics_rotamers.csv", package= "Bios2cor")

  #Creating dynamic_structure and rotamers objects
  wanted_frames <- seq(from = 5, to = 40, by = 10)
  dynamic_structure <- dynamic_structure(pdb, trj, wanted_frames)
  rotamers <- angle2rotamer(dynamic_structure, conversion_file)
  
  #Creating correlation and entropy objects
  wanted_residues <- c("F","H","N","Y","W")
  #dyn_corr <- dynamic_omes(dynamic_structure, rotamers, wanted_residues)
  dyn_corr <- dynamic_circular(dynamic_structure, wanted_residues)
  dyn_entropy <- dynamic_entropy(rotamers)
   
  #Writing results to csv file
  write.scores(dyn_corr, file, dyn_entropy )

}

\name{Bios2cor-package}
\Rdversion{2.2}
\alias{Bios2cor-package}
\alias{Bios2cor}
\alias{Bios2cor}
\docType{package}
\title{
Correlation/covariation Analysis in Biological Sequences and Simulations
}
\description{
The \code{Bios2cor} package is dedicated to the computation and analysis of correlation/covariation between positions in multiple sequence alignments (MSA) and between side chain dihedral angles during molecular dynamics simulations (MD). Features include the computation of correlation/covariation scores using a variety of scoring functions and their analysis through a variety of tools including network representation and principal components analysis. In addition, several utility functions are based on the R graphical environment to provide friendly tools for help in data interpretation. 

For clarity purpose, version 2 of the package differentiates scoring functions working on MSA and on MD because their arguments are different. Analysis functions are common with auto detection of MSA or MD.   

  The main functionalities of \code{Bios2cor} are summarized below:
  \describe{
    \item{(1) CORRELATION/COVARIATION METHODS : }{
    Methods that can be used to analyze sequence alignments and molecular simulations and to calculate a correlation/covariation matrix containing a score for each pair of positions (sequence alignment) or each pair of dihedral angles (molecular simulations).
    
    Methods working with sequence alignments (fasta or msf file is required):
      \itemize{
	\item \code{\link{omes}}: calculates the difference between the observed and expected occurrences of each possible pair of amino acids (x, y) at positions i and j of the alignment.
	
	\item \code{\link{mi}} and \code{\link{mip}}: calculate a score based on the probability of joint occurrence of events (MI)  and a corrected score by substraction of the average product (MIP), respectively. 

	\item \code{\link{elsc}}: calculates a score based on rigorous statistics of correlation/covariation in a perturbation-based algorithm. It measures how many possible subsets of size n would have the composition found in column j.

	\item \code{\link{mcbasc}}: relies on a substitution matrix giving a similarity score for each pair of amino acids.
	
      }
      
      Methods working with molecular simulations (pdb and dcd files are required) :
      \itemize{
	\item \code{\link{dynamic_circular}}: calculates a correlation/covariation score based on a circular version of the Pearson correlation coefficient, between each pair of side chain dihedral angles in a trajectory obtained from molecular dynamics simulations.
	
	\item \code{\link{dynamic_omes}}: calculates the difference between the observed and expected occurrences of each possible pair of rotamers (x, y) occuring at side chain dihedral angles i and j in a trajectory.
	
	\item \code{\link{dynamic_mi}} and \code{\link{dynamic_mip}}: calculate a score based on the probability of joint occurrence of rotameric states (MI) and a corrected score by substraction of the average product (MIP), respectively.

      }
 
      The methods working with molecular simulations require the following functions :
      \itemize{

	\item \code{\link{dynamic_structure}}: using the result of the \code{xyz2torsion} function from the \code{bio3D} package, creates a structure that contains side chain dihedral angle informations for each selected frame of the trajectory.

 	\item \code{\link{angle2rotamer}}: using the result of the \code{\link{dynamic_structure}} function, creates a structure that associates rotameric state to each side chain dihedral angle for each selected frame of the trajectory.

      }

   }
    
    \item{(2) ADDITIONNAL FUNCTIONS  : }{
    Functions that can be used to analyse the results of the correlation/covariation methods :
      
      Entropy functions :
      \itemize{
	\item \code{\link{entropy}}: calculates an entropy score for each position of the alignment. This score is based on sequence conservation and uses a formula derived from the Shannon's entropy.
	
	\item \code{\link{dynamic_entropy}}: calculates a "dynamic entropy" score for each side chain dihedral angle of a protein during molecular simulations. This score is based on the number of rotameric changes of the dihedral angle during the simulation.    
      }
      
      Filters:
      \itemize{
	\item \code{\link{delta_filter}}: given an entropy object, returns a delta filter for each position of the alignment or each side chain dihedral angle of the protein, based on entropy/dynamic entropy value.
      }
      
      PCA :
      \itemize{
	\item \code{\link{centered_pca}}: returns a principal component analysis of the double-centered correlation/covariation matrix passed as a parameter. A delta filter can be precised.
      }
    }
    
    \item{(3) OUTPUT FILES : }{
      Functions that can be used to produce output files.
       
      Some data structures can be stored in txt/csv files :
      \itemize{
	\item \code{\link{write.scores}}: Using the result of a correlation/covariation method, creates a file containing the score of each pair of positions (sequence alignment analysis) or of side chain dihedral angles (molecular simulations) and optionaly their entropy/dynamic_entropy score.

	\item \code{\link{top_pairs_analysis}}: Using the result of a correlation/covariation method and an integer N, creates two files containing (1) the top N pairs with their scores and (2) the individual elements, their contact counts and their entropy score for the top N pairs. Subsequently, these files can be  used for network visualization with the Cytoscape program accessible at \emph{https://cytoscape.org}. 
 
	\item \code{\link{write.pca}}: Using the result of the \code{\link{centered_pca}} function, creates a file that contains the coordinates of each element in the principal component space.
 
	\item \code{\link{write.pca.pdb}}: Using the result of the \code{\link{centered_pca}} function, creates a pdb file with the PCA coordinates on three principal components along with a pml file for nice visualization with the Pymol molecular visualization program accessible at \emph{https://pymol.org}.
      }
      
      Some data can be visualized in png/pdf files:
      \itemize{
	\item \code{\link{scores.boxplot}}: Using the result of one or several correlation/covariation methods, creates a boxplot to visualize the distribution of the Z-scores.

	\item \code{\link{network.plot}}: Using the result of the \code{\link{top_pairs_analysis}} function, creates the graph of a network representation of the data.

	\item \code{\link{scores_entropy.plot}}: Using the result of a correlation/covariation method and an entropy structure, creates a graph comparing correlation/covariation scores with entropy values. Each pair of elements (i,j) is placed in the graph with (entropy[i] ; entropy[j]) as coordinates. The color code of each point is based on its correlation/covariation score (red/pink color for top values, blue/skyblue for bottom values).

	\item \code{\link{pca_screeplot}}: Using the result of the \code{\link{centered_pca}} function, creates the graph of the eigen values (positive values only).

	\item \code{\link{pca_2d}}: Using the result of the \code{\link{centered_pca}} function, creates a graph with the projection of the elements on two selected components.

	\item \code{\link{angles.plot}}: Using pdb and dcd files and the result of a correlation/covariation method, creates graphs to monitor the time evolution of each dihedral angle in the top N pairs
      }
      
    }
  }
}
\details{
\tabular{ll}{
  Package: \tab BioCor\cr
  Type: \tab Package\cr
  Version: \tab 2.1\cr
  Date: \tab 2020-01-30\cr
  License: \tab GPL \cr
}
}
\author{
  Bruck TADDESE [aut], Antoine Garnier [aut], Madeline DENIAUD [aut], Lea BELLANGER [ctb], Julien PELE[ctb], Jean-Michel BECU [ctb], Marie CHABBERT [cre].
  Maintainer: Marie CHABBERT <marie.chabbert@univ-angers.fr>
}


\examples{
  #File path for output files
  wd <- tempdir()
  #wd <- getwd()
  file <- file.path(wd,"test_seq1") 

  #Importing multiple sequence alignment
  msf <- system.file("msa/toy_align.msf", package = "Bios2cor")
  align <- import.msf(msf)

  #Creating correlation object with OMES method
  omes <- omes(align, gap_ratio = 0.2)

  #Creating entropy object
  entropy <- entropy(align, gap_ratio = 0.2)

  #Creating delta filter based on entropy
  filter <- delta_filter(entropy, Smin = 0.3, Smax = 0.8)
  
  #Selecting a correlation matrix
  omes <-omes$score

  # Creating PCA object for selected correlation matrix and storing eigen values in csv file
  pca <- centered_pca(omes, filepathroot= file, pc= NULL, dec_val= 5, filter= filter)
  
  
}

\name{write.pca}

\Rdversion{1.1}

\alias{write.pca}

\title{
  Creates a file of coordinates in PCA space 
}

\description{
  Given an object of class 'pca' (result of the \code{\link{centered_pca}} function), stores the coordinates of each element in the PC space in a txt file
}

\usage{
  write.pca(corr_pca,filepathroot, pc=NULL, entropy= NULL)
}

\arguments{
 \item{corr_pca}{
  An object created by the \code{\link{centered_pca}} function from a correlation/covariation matrix
 }
 \item{filepathroot}{
  The root for the full path name of the output file where all coordinates on all components are stored. If NULL, a "PCA_COORD.csv" file is created in tempdir(). If not NULL, the "_PCA_COORD.csv extension is added to the filepathroot.
 }
 \item{pc}{
  An integer corresponding to the number of principal components for which coordinates of the elements are saved. By default, this number corresponds to the number of components with positive eigen values.
 } 
 \item{entropy}{
  An object created by the \code{\link{entropy}} or the \code{\link{dynamic_entropy}} function. DEfault is NULL. 
 }
}

\details{
  The object returned by the \code{\link{centered_pca}} function contains coordinates in the PC space for each element.
  Each line of the pca file will contain the name of the current element and its coordinates.
  Any line that contains Na value for X, Y or Z coordinates will be ignored.
}

\value{
  returns a file containing the coordinates of each element in PC space.
}

\author{
  Antoine GARNIER
}

\examples{
  #File path for output files
  out <- tempdir()
  file <- file.path(out,"test_seq4") 

  #Importing MSA file
  msf <- system.file("msa/toy_align.msf", package = "Bios2cor")
  align <- import.msf(msf)

  #Creating OMES correlation object and selecting correlation matrix
  omes <- omes(align, gap_ratio = 0.2)
  omes <-omes$Zscore

  #Creating PCA object for selected matrix
  pca <- centered_pca(omes, filepathroot= file, filter = NULL, pc= NULL, dec_val= 5)
   
  #Saving coordinates of elements in csv file 
  write.pca(pca, file, pc = 10, entropy = NULL)
  
}

\name{elsc}

\Rdversion{1.1}

\alias{elsc}

\title{
   Explicit Likelihood of Subset Covariation (ELSC) function
}

\description{
  Calculates a score based on rigorous statistics of correlation/covariation in a perturbation-based algorithm. It measures how many possible subsets of size n would have the composition found in column j in the subset alignment defined by the perturbation in column i, and in the ideal subset
(i.e., in a subset with the amino acid distribution equal to the total alignment).
}

\usage{
  elsc(align, gap_ratio = 0.2)
}

\arguments{
 \item{align}{
  An object of class 'align' created by the \code{\link{import.msf}} or the \code{\link{import.fasta}} function from a sequence alignment
 }
 \item{gap_ratio}{
  Numeric value between 0 and 1 indicating the maximal gap ratio at a given position in the MSA for this position to be taken into account. Default is 0.2, positions with more than 20 percent of gaps will not be taken into account in the analysis. When gap_ratio is 1 or close to 1, only positions with at least 1 aa are taken into account (positions with only gaps are excluded). 
 }
}

\details{
  The ELSC score at position [i,j] has been computed with the following formula :
  
  \deqn{ELSC(i,j) = -ln\prod_{y}^{ } \frac{{{N_{y(j)}}\choose{n_{y(j)}}}}{{{N_{y(j)}}\choose{m_{y(j)}}}}}
  
  As a reminder, a binomial coefficient \eqn{{N}\choose{k}} is computed as follow :
  \deqn{{{N}\choose{k}} = \frac{N!}{k!(N-k)!}}
  
  where :
    \itemize{
      \item {\eqn{N_{y(j)}}} {is the number of residues y at position j in the total (unperturbed) sequence alignment}
      \item {\eqn{n_{y(j)}}} {is the number of residues y at position j in the subset alignment defined by the perturbation in column i}
      \item {\eqn{m_{y(j)}}} {is the number of residues y at position j in the ideal subset (i.e., in a subset with the amino acid distribution equal to the total alignment)}
    }
}

\value{
  A list of two elements which are numeric matrices containing the ELSC scores and Z-scores for each pair of elements. 
}

\author{
  Madeline DENIAUD and Marie CHABBERT
}


\references{
 Dekker JP, Fodor A, Aldrich RW, Yellen G. A perturbation-bqsed method for calculating explicit likelihood of evolutionary covariance in multiple sequence alignements. Bioinformatics 2004;20:1565-1572.
 }



\examples{
  #Importing MSA file 
  align <- import.msf(system.file("msa/toy_align.msf", package = "Bios2cor"))

  #Creating correlation object with ELSC method for positions with gap ratio < 0.1
  elsc <- elsc(align, gap_ratio = 0.1)

  #Creating correlation object with ELSC method for positions with gap_ratio < 0.2 (Default)
  #elsc <- elsc(align)
}

\name{write.pca.pdb}

\Rdversion{1.1}

\alias{write.pca.pdb}

\title{
  PDB and PML file creation for 3D representation of PCA analysis
}

\description{
  Given a PCA structure, creates .pdb and .pml files for 3D visualization with Pymol
}

\usage{
  write.pca.pdb(corr_pca, filepathroot, trio_comp= c(1:3))
}

\arguments{
 \item{corr_pca}{
   An object created by the \code{\link{centered_pca}} function from a correlation/covariation matrix
 }
 \item{filepathroot}{
   The root for the full path name of the output PDB and PML files. If NULL, two PCA_l_m_n.pdb and PCA_l_m_n.pml files are created in tempdir(). If not null, '_PCA_l_m_n.pdb' and '_PCA_l_m_n.pml' extensions are added to the filepathroot.  The l, m, n parameters are the 3 selected components. 
 }
 \item{trio_comp}{
  A numeric vector of length 3 indicating the principal components to be displayed. Default is c(1, 2, 3). 
 }
}

\details{
  This function creates PDB and PML files to visualize the positions of the elements (sequence positions or dihedral angles) in a 3D space corresponding to three selected components of the PCA space. The PDB file can be viewed in any molecular graphics softaware. The PML file allows a nice representation with Pymol (axis, background color, size of points and for GPCRs, color code for helices).
}

\value{
  Returns two files:  a PDB file which contains three PCA coordinates for each element in PDB format and a PML file for nice visualization with Pymol. 
}

\author{
  Antoine GARNIER and Marie CHABBERT
}

\examples{

# Example for MSA
  #File path for output files
  out <- tempdir()
  file <- file.path(out,"test_seq5") 

  #Importing MSA file
  align <- import.msf(system.file("msa/toy_align.msf", package = "Bios2cor"))

  #Creating OMES correlation object and selecting correlation matrix
  omes <- omes(align, gap_ratio = 0.2)
  cor_omes <- omes$Zscore
 
  #Creating PCA object for selected  matrix 
  pca <- centered_pca(cor_omes, filepathroot = file, filter = NULL, pc = NULL, dec_val = 5)

  #Creating PDB and PML files (open PDB file with Pymol then "File > Run" PML file)
  indices <- c(1,2,3)
  write.pca.pdb(pca, file, indices)


 
### Example for MD
  #File path for output files
  wd <- tempdir()
  #wd <-getwd() 
  file <- file.path(wd,"test_dyn5") 

  #Redaing pdb and dcd files
  pdb <- system.file("rotamer/toy_coordinates.pdb", package= "Bios2cor")
  trj <- system.file("rotamer/toy_dynamics.dcd", package= "Bios2cor")

  #Creating dynamic_structure object for selected frames
  nb_frames_wanted <- 40
  wanted_frames <- seq(from = 5, to= nb_frames_wanted, by = 10)
  dynamic_structure <- dynamic_structure(pdb, trj, wanted_frames)

  #Creating rotamers object
  conversion_file <- system.file("rotamer/dynameomics_rotamers.csv", package= "Bios2cor")
  rotamers <- angle2rotamer(dynamic_structure, conversion_file)
  
  #Creating OMES correlation object and selecting correlation matrix
  wanted_residues <- c("W","Y","F","N")
  omes <- dynamic_omes(dynamic_structure, rotamers, wanted_residues)
  cor_omes <- omes$Zscore_noauto

  #Creating PCA object for selected matrix 
  pca <- centered_pca(cor_omes, file, filter = NULL, pc = NULL, dec_val = 5)

  #Creating PDB and PML files (open PDB file with Pymol then "File > Run" PML file)
  indices <- c(1,2,3)
  write.pca.pdb(pca, file, indices)

}

\name{pca_screeplot}

\Rdversion{1.1}

\alias{pca_screeplot}

\title{
  Creates PCA screeplot 
}

\description{
  Given a PCA structure (result of the \code{\link{centered_pca}} function), creates a screeplot of the positive eigen values
}

\usage{
  pca_screeplot(pca_struct, filepathroot=NULL)
}

\arguments{
 \item{pca_struct}{
  An object created by the \code{\link{centered_pca}} function
 }
 \item{filepathroot}{
  The root of the full path name for the output file. Default is NULL (a "EIGEN.png" file will be created). If not NULL, the filepathroot will have the "_EIGEN.png" extension. 
 }
}

\value{
  A screeplot of positive eigen values 
}

\author{
  Antoine GARNIER
}

\examples{
  #File path for output files
  wd <- tempdir()
  #wd <-getwd() 
  file <- file.path(wd,"test_seq3") 

  #Importing MSA file
  msf <- system.file("msa/toy_align.msf", package = "Bios2cor")
  align <- import.msf(msf)

  #Creating OMES correlation object
  omes <- omes(align, gap_ratio = 0.2)

  #Selecting correlation matrix
  omes <-omes$Zscore

  #Creating PCA object for selected correlation matrix and saving eigen values
  pca <- centered_pca(omes, filepathroot= file, pc= NULL, dec_val= 5, filter = NULL)

  #Plotting scree plot 
  pca_screeplot(pca, file)
  #pca_screeplot(pca)  
}

\name{random.msa}
\Rdversion{1.1}
\alias{random.msa}
\title{
Random Alignment
}
\description{
Builds a multiple sequence alignment (MSA) of random sequences.
}
\usage{
random.msa(nb.seq = 100, id = "SEQ", nb.pos = 100, gap = FALSE,
aa.strict = FALSE, align = NULL, align.replace = TRUE)
}
\arguments{
  \item{nb.seq}{
  a numeric value indicating the number of sequences in the random MSA. Default is 100. 
}
\item{id}{
  a string of characters used to tag each sequence name. Default is "SEQ". An
  incremented number is attached to this tag to name each sequence.
}
  \item{nb.pos}{
  a numeric value indicating the length of each sequence in the random MSA. Default is 100.
}
  \item{gap}{
  a logical value indicating whether the gap character should be considered as a supplementary symbol
  (TRUE) or not (FALSE). Default is FALSE. 
}
  \item{aa.strict}{
  a logical value indicating whether only strict amino acids should be taken into account (TRUE)
  or not (FALSE). Default is FALSE.
}
  \item{align}{
  an object of class 'align', obtained from \code{\link{import.fasta}} or \code{\link{import.msf}} function.
  If this parameter is not NULL, the composition of the output sequences is based on the composition of the input sequences. Default is NULL.
}
  \item{align.replace}{
  a logical value indicating random drawing with replacement (TRUE) or without replacement (FALSE) of characters present in \code{align}. Default is FALSE.
}
}
\details{
\code{random.msa} may be used to compare a reference MSA to a random MSA. The random MSA must have the same characteristics as the reference MSA (same number of sequences of same length). 

A procedure can be applied to the random MSA to assess the amount of variance due to random mutations in the reference MSA. 

The \code{subset} function is used for random selection of the amino acids. If a truly random procedure is needed, see \code{random} package.
}
\value{
A named list whose objects correspond to random sequences.
}
\note{
This function has been initially developped in the \code{bios2mds} R package (Julien PELE [aut], Marie CHABBERT [cre]).
}
\author{
Julien PELE
}
\references{
For an application of random MSA see : 

Pele J, Abdi H, Moreau M, Thybert D and Chabbert M (2011) Multidimensional scaling reveals the main evolutionary pathways of class A G-protein-coupled receptors. \emph{PLoS ONE} \bold{6}: e19094. doi:10.1371.
}
\seealso{
\code{permutation} and \code{synsequence} functions from \code{seqinr} package.
}
\examples{
  #Importing MSA file
  aln <- import.fasta(system.file("msa/toy2_align.fa", package = "Bios2cor"))

  #Generating a random sequence alignment with the same characterics as input file
  nb.seq <- length(aln)
  nb.pos <- length(aln[[1]])
  aln.random <- random.msa(nb.seq = nb.seq, nb.pos = nb.pos)
}

\name{dynamic_entropy}

\Rdversion{1.1}

\alias{dynamic_entropy}

\title{
Variability score
}

\description{
  Measures a "dynamic entropy" or variability score of each dihedral angle based on the number of rotameric changes during the molecular dynamics trajectory.
}

\usage{
  dynamic_entropy(rotamers)
}

\arguments{
 \item{rotamers}{
  A character matrix of type 'rotamers' that is produced by the \code{angle2rotamer} function. 
  The matrix indicates the rotameric state of each side chain dihedral angle for each frame of the trajectory.
 }
}

\details{
  The "dynamic entropy" score S is computed by summing the number of rotameric changes over all frames, normalized to the number of frames. It is not a "true entropy" score but gives usefull information on variability of the dihedral angle during the MD simulation.
}

\value{
  A numeric vector containing a "dynamic entropy" score for each side chain dihedral angle during the trajectory. The score is comprised between 0 (no change in the rotameric state during the trajectory) and 1 (rotameric change for every frame of the trajectory).   
}

\author{
 Antoine GARNIER, Lea BELLENGER and Marie CHABBERT 
}

\examples{
  #Reading pdb and dcd files
  pdb <- system.file("rotamer/toy_coordinates.pdb", package= "Bios2cor")
  trj <- system.file("rotamer/toy_dynamics.dcd", package= "Bios2cor")
 
  #Reading conversion file
  conversion_file <- system.file("rotamer/dynameomics_rotamers.csv", package= "Bios2cor")

  #Creating the dynamic_structure and rotamers objects
  wanted_frames <- seq(from = 1, t = 40, by = 5)
  dynamic_structure <- dynamic_structure(pdb, trj, wanted_frames)
  rotamers <- angle2rotamer(dynamic_structure, conversion_file)
 
  #creating the dynamic_entropy object
  dynamic_entropy <- dynamic_entropy(rotamers)
}

\name{angle2rotamer}

\Rdversion{1.1}

\alias{angle2rotamer}

\title{
  Converts dihedral angles to rotamers
}

\description{
  Given an object of class 'structure' and an angle to rotamer conversion file, associates a rotamer to each dihedral angle value.
  The object of class 'structure' contains dihedral angle values for each side chain dihedral angle and each frame of the trajectory.
  The conversion file is a reference file that contains the rotamer to be associated with a dihedral angle value, depending on the residue type and the dihedral angle considered. This function will allow to compute correlation/covariation scores between rotameric states.
}

\usage{
  angle2rotamer(dynamic_structure, 
    conversion_file=system.file("rotamer/dynameomics_rotamers.csv", package= "Bios2cor"))
}

\arguments{
 \item{dynamic_structure}{
   Dihedral angle structure, result of the \code{dynamic_structure} function
 }
 \item{conversion_file}{
   The file containing the rotamer to be associated with each residue dihedral angle depending of the dihedral angle value.

   Each line contains five fields, separated by ','. The five fields represent the amino acid name ("R", "N",...), the dihedral angle name("chi1", "chi2",...), the associated rotamer ("g+", "t", "g-"), the start and stop angles (between -180 and 180). For example, for the \emph{chi1} angle of the valine residue, a torsion angle between 0 and 120 is associated to rotameric state \emph{g+}.

   Default is the "rotamer/dynameomics_rotamers.csv" conversion file provided with the Bios2cor package from the dynameomics database (\code{http://dynameomics.org}) .
 }
}

\details{
  In the torsion object and in the conversion file, dihedral angle values vary between -180 and 180.
}

\value{
  A character matrix containing the rotameric state of each side chain dihedral angle for each frame in the trajectory, depending on the dihedral angle value and on a conversion file.
}

\author{
  Antoine GARNIER and Lea BELLENGER
}

\references{
 Van der Kamp MW, Schaeffer RD, Jonsson AL, Scouras AD, Simms AM, Toofanny RD, Benson NC, Anderson PC, Merkley ED, Rysavy S, Bromley D, Beck DAC, and Daggett V. Dynameomics: A comprehensive database of protein dynamics. Structure, 18: 423-435, 2010. 
}


\examples{
  #Reading pdb and dcd files
  pdb <- system.file("rotamer/toy_coordinates.pdb", package= "Bios2cor")
  trj <- system.file("rotamer/toy_dynamics.dcd", package= "Bios2cor")
 
  #Reading conversion file 
  #conversion_file <- system.file("rotamer/dynameomics_rotamers.csv", package= "Bios2cor")

  #Creating dynamic_structure object
  wanted_frames <- seq(from = 1, to = 40, by = 2)
  dynamic_structure <- dynamic_structure(pdb, trj, wanted_frames)
  
  #Creating rotamers object with default conversion_file
  rotamers <- angle2rotamer(dynamic_structure)
  
  #Creating rotamers object with conversion_file   
  #rotamers <- angle2rotamer(dynamic_structure, conversion_file)
 
}

\name{centered_pca}

\Rdversion{1.1}

\alias{centered_pca}

\title{
  Performs principal component analysis of a correlation/covariation matrix 
}

\description{
  Given a correlation/covariation matrix, performs the principal component analysis of the centered matrix.
}

\usage{
  centered_pca(corr_matrix, filepathroot, filter = NULL, pc= NULL, dec_val= 5)
}

\arguments{
 \item{corr_matrix}{
  A score or Zscore matrix created by a correlation/covariation function (\code{\link{omes}}, \code{\link{mip}}, \code{\link{elsc}}, \code{\link{mcbasc}}, \code{\link{dynamic_circular}}, \code{\link{dynamic_omes}}, \code{\link{dynamic_mip}})
 }
 \item{filepathroot}{
  The root of the full path name for the csv and png files where eigen values are stored or displayed. By default, two "EIGEN.csv" and "EIGEN.png" files are created in temp(dir). If not NULL, the extensions "_EIGEN.csv" and "_EIGEN.png" are added to the filepathroot. 
 }
 \item{filter}{
  A filter calculated by the \code{\link{delta_filter}} function to limit the analysis to elements within a given entropy/dynamic_entropy range. DEFAULT is NULL (no filter is applied and each element has the same weight equal to the inverse of the number of elements).
 }
 \item{pc}{
  A numeric value indicating the number of principal components to be saved. Default is NULL (all the principal components are saved).
 }
 \item{dec_val}{
  A numeric value corresponding to the precision when the \emph{round} function is used. Default is 5.  
 }
}

\details{
  This function performs a principal component analysis of a correlation/covariation matrix after double centering. It is based on the matrix centering algorithm of the \code{mmds.R} function from the \code{Bios2mds} package. The elements have the same weight except when a delta filter is indicated. In this latter case, only the elements allowed by the delta filter are taken into account.
}

\value{
  returns an object of class 'pca' which is a named list of four elements: 
  \item{eigen}{
  a numeric vector of the eigenvalues
  }
  \item{eigen.perc}{
  a numeric vector of the relative eigenvalues (eigenvalues divided by the sum of the absolute eigenvalues)
  }
  \item{coord}{
  a numeric matrix representing the coordinates of each element of the correlation/covariation matrix in the PCA space
  }
  \item{source}{
  a named list with 2 elements, the correlation/covariation matrix (\code{cor}) and the delta filter vector (\code{filter}) to limit the analysis to elements within a given entropy range.
  }
 returns also two files: a csv file containing eigen values and a png file displaying eigen values.

}



\author{
  Antoine GARNIER and Marie CHABBERT 
}

\examples{
  #File path for output files
  out <- tempdir()
  file <- file.path(out,"test_seq_pca") 

  #Importing multiple sequence alignment
  msf <- system.file("msa/toy_align.msf", package = "Bios2cor")
  align <- import.msf(msf)

  #Creating correlation object with OMES method
  omes <- omes(align, gap_ratio= 0.2)

  #Creating entropy object
  entropy <- entropy(align, gap_ratio=0.2)

  #Creating delta filter based on entropy
  filter <- delta_filter(entropy, Smin = 0.2, Smax = 0.6)
  
  #Selecting a correlation/covariation matrix
  matrix_omes <-omes$score

  #Creating PCA object for selected matrix and storing eigen values in csv file
  pca <- centered_pca(matrix_omes, filepathroot= file, filter = filter)

}

\name{delta_filter}

\Rdversion{1.1}

\alias{delta_filter}

\title{
  Creation of an entropy/dynamic score delta filter for each element
}

\description{
  Given an entropy object (result of the \code{\link{entropy}} or of the \code{\link{dynamic_entropy}} function), creates a vector with a delta filter of each element based on the entropy value. The vector will be used to limit the analysis to the elements in the given entropy range in the \code{\link{centered_pca}} and \code{\link{top_pairs_analysis}} functions.
}

\usage{
  delta_filter(entropy, Smin = 0, Smax = 1)
}

\arguments{
 \item{entropy}{
  An object created by the \code{\link{entropy}} or the \code{\link{dynamic_entropy}}function
 }
 \item{Smin}{
  A value indicating the minimum entropy value. (Smin = 0 by default)
 }
 \item{Smax}{
  A value indicating the maximum entropy value. (Smax = 1 by default)
 }
}

\details{
  The object returned by the \code{\link{entropy}} or the \code{\link{dynamic_entropy}} function contains an entropy score for each element.
  The delta weighting of each element is calculated as follow :
      \deqn{ weighting[i] = \left\{ \begin{array}{ll}  1, & Smin < entropy[i] < Smax \\  0, & otherwise \end{array}  \right. }
}

\value{
  A vector that contains a 0 or 1 weighting score for each element (position in sequence alignment or side chain dihedral angle in trajectory) to limit principal component and top pair analysis to elements within a given entropy range.
}

\author{
  Antoine GARNIER
}

\examples{
  #Importing MSA
  align <- import.msf(system.file("msa/toy_align.msf", package = "Bios2cor"))

  #Creating entropy object
  entropy <- entropy(align)
  
  #Creating delta filter based on entropy
  filter <- delta_filter(entropy, Smin = 0.4, Smax = 0.6)
  
}

\name{network.plot}

\Rdversion{1.1}

\alias{network.plot}

\title{
  Creates network structure of top elements
}

\description{
  Given a top_pairs object (result of the \code{\link{top_pairs_analysis}} function), creates a network to visualize the elements involved in the top scoring pairs and their links
}

\usage{
  network.plot(top_pairs, filepathroot)
}

\arguments{
 \item{top_pairs}{
  An object of class 'top_pairs' created by the \code{\link{top_pairs_analysis}} function
 }
 \item{filepathroot}{
  The root of the full path name for the output file. If NULL, a "NETWORK.pdf" file will be created in tempdir(). If not NULL, the filepathroot will have the "_NETWORK.pdf" extension. 
 }
}

\value{
  A network representing links between elements in the top scoring pairs
}

\author{
  Antoine GARNIER
}

\examples{
  #File path for output file
  out <- tempdir()
  file <- file.path(out,"test_omes") 

  #Importing MSA file
  msf <- system.file("msa/toy_align.msf", package = "Bios2cor")
  align <- import.msf(msf)

  #Creating OMES correlation object for positions with gap ratio < 0.2
  omes <- omes(align, gap_ratio= 0.2)
  
  #Selecting a correlation matrix
  omes <-omes$Zscore
   
  #Analyzing pairs with top scores and creating 'top_pairs' object
  top_pairs <- top_pairs_analysis(omes, top = 25, file)

  #Plotting the network structure of top pairs in pdf file 
  network.plot(top_pairs, file)
}

\name{scores_entropy.plot}

\Rdversion{1.1}

\alias{scores_entropy.plot}

\title{
 Scores versus entropy graph
}

\description{
  The aim of this graph is the visualization of the entropy values of the elements in top and bottom pairs. 
}

\usage{
  scores_entropy.plot(entropy, corr_matrix, 
    filepathroot, 
    elite=25, 
    high=275, 
    filter=NULL)
}

\arguments{
 \item{entropy}{
  An object created by the \code{\link{entropy}} (sequence alignment) or the \code{\link{dynamic_entropy}} (trajectory) function. 
 }
 \item{corr_matrix}{
  A correlation/covariation matrix created by one of the correlation/covariation functions: 
 }
  \item{filepathroot}{
  The root of the full path name of the graph that will be created. If NULL, a "ei_ej.pdf" file will be created in tempdir(). If not NULL, the filepathroot will have the "_ei_ej.pdf" extension. 
 }
 \item{elite}{
  An integer to determine the number of pairs with the highest and lowest scores (e.g. 25: pairs ranked 1 to 25 in decreasing or increasing order) to be colored with the "elite" color codes. Default is 25.
 }
 \item{high}{
  An integer to determine the number of pairs with the next highest and lowest scores (e.g. 275: pairs ranked 26 to 275 in decreasing or increasing order) to be colored with the "high" color codes. Default is 275.
 }
 \item{filter}{
  A vector created by the \code{\link{delta_filter}} function, to limit the analysis to elements within a given entropy range. When filter is not NULL, only top elite and high pairs are visualized. Default is NULL. 
 }
}

\details{
  Using the result of a correlation/covariation method and an entropy structure, creates a graph comparing correlation/covariation scores with entropy values. Each pair of elements (i,j) is placed in the graph with (entropy[i] ; entropy[j]) as coordinates. In the absence of filter, the color code of each point is based on its correlation/covariation score (dark and light blue for top elite and high values, red and pink for bottom elite and high values). In the presence of an entropy based filter, only top elite and high scores are visualized in dark and light blue, respectively,  
}

\value{
  A graph showing the entropy values of the elements in pairs with top and bottom entropy scores 
}

\references{
For an application of these graphs see : 

Pele J, Abdi H, Moreau M, Thybert D and Chabbert M (2011) Multidimensional scaling reveals the main evolutionary pathways of class A G-protein-coupled receptors. \emph{PLoS ONE} \bold{6}: e19094. doi:10.1371.
}


\author{
  Julien PELE, Antoine GARNIER and Marie CHABBERT
}

\examples{
  
##Example with MSA
  #File path for output file
  out <- tempdir()
  file <- file.path(out,"test_seq7") 
  
  #Importing MSA file  
  msf <- system.file("msa/toy_align.msf", package = "Bios2cor")
  align <- import.msf(msf)

  #Creating OMES correlation object and selecting a correlation matrix
  correlation <- omes(align, gap_ratio = 0.2)
  corr_matrix <- correlation$Zscore 

  #Creating entropy object
  entropy <- entropy(align)

  #Creating a delta filter based on entropy
  filter <- delta_filter(entropy, Smin = 0.4, Smax = 0.7)
 
  #Creating the entropy graph
  scores_entropy.plot(entropy, corr_matrix, filepathroot = file, filter=filter)

#Example with MD
 \donttest{
	 #File path for output file
	 wd <- tempdir()
	 #wd <-getwd() 
	 file <- file.path(wd,"test_dyn7") 

	 #Reading pdb and dcd files
	 pdb <- system.file("rotamer/toy_coordinates.pdb", package= "Bios2cor")
	 trj <- system.file("rotamer/toy_dynamics.dcd", package= "Bios2cor")

	 #Creating dynamic_structure object for selected frames 
	 wanted_frames <- seq(from = 1, to = 40, by = 1)
	 dynamic_structure <- dynamic_structure(pdb, trj, wanted_frames)
	   
	 #Creating rotamers object using conversion_file
	 conversion_file <- system.file("rotamer/dynameomics_rotamers.csv", package= "Bios2cor")
	 rotamers <- angle2rotamer(dynamic_structure, conversion_file)

	 #Creating the dynamic_entropy and filter objects 
	 entropy <- dynamic_entropy(rotamers)
	 filter <- delta_filter(entropy, Smin = 0.0, Smax = 0.1)

	 #Creating correlation object
	 #dyn_cor <- dynamic_circular(dynamic_structure)
	 dyn_cor <- dynamic_omes(dynamic_structure,rotamers) 

	 #selection correlation matrix
	 corr_matrix <- dyn_cor$Zscore_noauto

	 #Creating the entropy graph
	 scores_entropy.plot(entropy, corr_matrix, filepathroot = file, filter=filter)
}
}

\name{write.entropy}

\Rdversion{1.1}

\alias{write.entropy}

\title{
  Writes and displays entropy values
}

\description{
  Given an entropy/dynamic_entropy object, writes each element (position or dihedral angle) and its entropy/dynamic_entropy value in a csv file, and displays the histogram. 
}

\usage{
  write.entropy(entropy, filepathroot = NULL)
}

\arguments{
 \item{entropy}{
  An object created by the \code{\link{entropy}} or the \code{\link{dynamic_entropy}} function.
 }
 \item{filepathroot}{
  The root of the full path name for the entropy output file. Default is NULL (a "ENTROPY.csv" file is created). If filepathroot is not NULL, a "_ENTROPY.csv" extension is added to the file path root. 
 }
}

\details{
  The \code{\link{entropy}} function calculates entropy score for each position of an alignment file. The \code{\link{dynamic_entropy}}function calculate a "dynamic entropy" score for each side chain dihedral angle of a protein during a molecular simulations. 
}


\value{
  A csv file containing the elements and their scores. A png file displaying the histogram of the scores.
}

\author{
  Antoine GARNIER
}

\examples{
  #File path for output files
  wd <- tempdir()
  #wd <-getwd() 
  file <- file.path(wd,"test_seq") 

  #Importing multiple sequence alignment
  align <- import.msf(system.file("msa/human_gpcr.msf", package = "Bios2cor"))

  #Creating entropy object
  entropy <- entropy(align, gap_ratio = 0.2)
  
  #Writing results to csv file
  write.entropy(entropy, file)
}


 

\name{entropy}

\Rdversion{1.1}

\alias{entropy}

\title{
Entropy score
}

\description{
  Measures the entropy score of each position in a sequence alignment
}

\usage{
  entropy(align, gap_ratio=0.2)
}

\arguments{
 \item{align}{
  An object created by the \code{\link{import.msf}} or the \code{\link{import.fasta}} function from a multiple sequence alignment file
 }
 \item{gap_ratio}{
  Numeric value between 0 and 1 indicating the maximal gap ratio at a given position in the MSA for this position to be taken into account. 1 is excluded (positions with gaps only). Default is 0.2, positions with more than 20 percent of gaps will not be taken into account in the analysis. When gap_ratio is 1 or close to 1, only positions with at least 1 aa are taken into account (positions with only gaps are excluded). 
 }

}

\details{
  The entropy score S at position i has been computed with a formula derived from the Shannon's entropy as follow : 
  \deqn{{S(i)} = - \sum_{x}^{ } {p} _{x}(i) \log_{20} {p} _{x}(i)}

  where :
    \itemize{
      \item {\emph{i}  is the position in the sequence}
      \item {\emph{x}  is the sequence index}
      \item {\emph{ \eqn{{p} _{x}(i)}  }  represents the frequency of residue x at position i}
    }
}

\value{
  A vector containing an entropy value for each position in the alignment
}

\author{
  Antoine GARNIER and Marie CHABBERT
}


\references{
  Shannon CE. A mathematical theory of communication. Bell Syst Techn J 1948;27:379-423.
} 
  
\examples{
  #Importing MSA file
  align <- import.msf(system.file("msa/human_gpcr.msf", package = "Bios2cor"))

  #creating entropy object for positions with gap ratio < 0.5
  entropy <- entropy(align,gap_ratio=0.5)
}

\name{mi}

\Rdversion{1.1}

\alias{mi}

\title{
  Mutual Information (MI) function
}

\description{
  Calculates a mutual information score (MI) based on the probability of joint occurrence of events.
}

\usage{
  mi(align, gap_ratio= 0.2)
}

\arguments{
 \item{align}{
  An object of class 'align' created by the \code{\link{import.msf}} or the \code{\link{import.fasta}} function from a sequence alignment
 }
 \item{gap_ratio}{
  Numeric value between 0 and 1 indicating the maximal gap ratio at a given position in the MSA for this position to be taken into account. Default is 0.2, positions with more than 20 percent of gaps will not be taken into account in the analysis. When gap_ratio is 1 or close to 1, only positions with at least 1 aa are taken into account (positions with only gaps are excluded). 
 }
}

\details{
  The MI score at position [i,j] has been computed with the following formula :
  
    \itemize{
      \item {\emph{\eqn{{MI(i,j) = \sum_{x,y}^{ } p_{x,y}(i,j) ln\frac{p_{x,y}(i,j)}{p_{x}(i)p_{y}(j)}}}}} {}
    }
    
  and where \emph{\eqn{p_{x,y}(i,j)}} is the frequency of the amino acid pair (x,y) at positions i and j.
      
  N.B. this formula has been widely applied in the field of sequence correlation/covariation but favors pairs with high entropy.
}

\value{
  A list of two elements which are numeric matrices containing the MI scores and Z-scores for each pair of elements. 
}

\author{
  Madeline DENIAUD and Marie CHABBERT
}


\references{
 Dunn SD, Wahl LM, Gloor GB. Mutual information without the influence of phylogeny or entropy dramatically improves residue contact prediction. Bioinfor;atics 2008;24:333-340.
 Martin LC, Gloor GB, Dunn SD, Wahl LM. Using infor;ation theory to search for co-evolving residues in proteins. Bioinformatics 2005;21:4116-4124.
}
  
  
\examples{
  #Importing MSA file
  align <- import.fasta(system.file("msa/toy2_align.fa", package = "Bios2cor"))

  #Creating correlation object with MI method for positions with gap ratio < 0.2 (Default)
  mi <- mi(align)
}

\name{dynamic_mi}

\Rdversion{1.1}

\alias{dynamic_mi}

\title{
  Mutual Information (MI) function applied to rotamers in molecular dynamics simulations
}

\description{
  Calculates a mutual information score (MI) based on the probability of joint occurrence of events.
}

\usage{
  dynamic_mi(
    dynamic_structure,
    rotamers,
    res_selection=
      c("C","I","L","M","V","R","H","K","D","E","N","Q","F","Y","W","T","S","P")
  )
}

\arguments{
 \item{dynamic_structure}{
   An object of class 'structure' that is created by the \code{\link{dynamic_structure}} function
 }
 \item{rotamers}{
  A character matrix of type 'rotamers' that is produced by the \code{\link{angle2rotamer}} function. 
  The matrix indicates the rotameric state of each side chain dihedral angle for each frame of the trajectory.
 }
 \item{res_selection}{
   List of amino acids that will be taken into account in the correlation/covariation matrix. By default, all the amino acids are taken into account except Gly and Ala, with no side chain dihedral angles.  
 }
}

\details{
  The MI score at position [i,j] has been computed with the following formula :
  
    \itemize{
      \item {\emph{\eqn{{MI(i,j) = \sum_{x,y}^{ } p_{x,y}(i,j) ln\frac{p_{x,y}(i,j)}{p_{x}(i)p_{y}(j)}}}}} {}
     }
    
 where \emph{\eqn{p_{x,y}(i,j)}} is the frequency of the rotamer pair (x,y) at dihedral angles i and j.
      
}

\value{
 returns a list of four elements which are numeric matrices containing (1) the correlation/covariation scores for each pair of rotamers (score), (2) the Z-scores for each pair of rotamers (Zscore), (3) the correlation/covariation scores for each pair of rotamers with zero values for autocorrelation (correlation within the same side chain) (score_noauto) and (4) the Z-scores calculated without autocorrelation pairs and zero values for autocorrelation pairs (Zscore_noauto). 
}

\author{
   Antoine GARNIER and Marie CHABBERT
}

\references{
 Dunn SD, Wahl LM, Gloor GB. Mutual information without the influence of phylogeny or entropy dramatically improves residue contact prediction. Bioinfor;atics 2008;24:333-340.
 Martin LC, Gloor GB, Dunn SD, Wahl LM. Using infor;ation theory to search for co-evolving residues in proteins. Bioinformatics 2005;21:4116-4124.
}

\examples{
  #Reading pdb and dcd files
  pdb <- system.file("rotamer/tiny_toy_coordinates.pdb", package= "Bios2cor")
  trj <- system.file("rotamer/tiny_toy_dynamics.dcd", package= "Bios2cor")

  #Creating dynamic_structure object
  wanted_frames <- seq(from = 5, to = 40, by = 15)
  dynamic_structure <- dynamic_structure(pdb, trj, wanted_frames)

  #Creating rotamers object using conversion_file
  conversion_file <- system.file("rotamer/dynameomics_rotamers.csv", package= "Bios2cor")
  rotamers <- angle2rotamer(dynamic_structure, conversion_file)

  #Creating correlation object for selected residues using MI method
  wanted_residues <- c("H","N")
  mi_corr <- dynamic_mi(dynamic_structure, rotamers, wanted_residues)
}

\name{omes}

\Rdversion{1.1}

\alias{omes}

\title{
  OMES (Observed minus Expected Squared) function
}

\description{
  Calculates the difference between the observed and expected occurrences of each possible pair of amino acids (x, y) at positions i and j of the alignment
}

\usage{
  omes(align, gap_ratio = 0.2)
}

\arguments{
 \item{align}{
  An object of class 'align' created by the \code{\link{import.msf}} or the \code{\link{import.fasta}} function from a sequence alignment
 }
 \item{gap_ratio}{
  Numeric value between 0 and 1 indicating the maximal gap ratio at a given position in the MSA for this position to be taken into account. Default is 0.2, positions with more than 20 percent of gaps will not be taken into account in the analysis. When gap_ratio is 1 or close to 1, only positions with at least 1 aa are taken into account (positions with only gaps are excluded). 
 }
}

\details{
  The OMES score at position [i,j] has been computed with the following formula :
  
  \deqn{{OMES(i,j)} = \frac{1}{N(i,j)} \sum_{x,y}^{ }(N_{x,y}^{obs}(i,j)-N_{x,y}^{ex}(i,j))^2}
  
  with : \emph{\eqn{N_{x,y}^{ex}(i,j) = p_{x}(i)p_{y}(j)N}}

  where :
    \itemize{
      \item {\emph{\eqn{N_{x,y}^{obs}(i,j)}}  is number of times that each (x,y) pair is observed at positions i and j}
      \item {\emph{\eqn{N_{x,y}^{ex}(i,j)}}  is number of times that each (x,y) pair would be expected, based on the frequency of residues x and y at positions i and j}
      \item {\emph{\eqn{N(i,j)}}  is the number of sequences in the alignment with non-gapped residues at positions i and j}
      \item {\emph{\eqn{p_{x}(i)}}  is the frequency of amino acid x at position i}
      \item {\emph{\eqn{p_{y}(j)}}  is the frequency of amino acid y at position j}
    }
}

\value{
  A list of two elements which are numerical matrices containing the OMES scores and Z-scores for each pair of elements. 
}

\author{
  Jean-Miche BECU, Madeline DENIAUD, and Marie CHABBERT
}

\references{
Fodor AA and Aldrich RW. Influence of conservation on calculations of amino acid covariance in multiple
sequence alignments. Proteins. 2004;56(2):211-21.
}

\examples{
  #Importing MSA file
  msf <- system.file("msa/toy_align.msf", package = "Bios2cor")
  align <- import.msf(msf)

  #Creating correlation object with OMES method for positions with gap ratio < 0.2 (Default)
  omes <- omes(align, gap_ratio = 0.2)
 }

\name{dynamic_structure}

\Rdversion{1.1}

\alias{dynamic_structure}

\title{
  Creates the data structure for the analysis of side chain dihedral angles   
}

\description{
  Given a structure pdb file, a trajectory dcd file and frame indices, gathers information on side chain dihedral angles in a unique structure. This structure will be used in correlation/covariation methods aimed at analyzing side chain rotational motions during molecular dynamics simulations.
}

\usage{
  dynamic_structure(pdb, trj, frames=NULL)
}

\arguments{
 \item{pdb}{
   Filepath of the  pdb file
 }
 \item{trj}{
   Filepath of trajectory file (dcd file)
 }
 \item{frames}{
   Indicates the selected frames for the analysis, created with the \code{seq} function (Usage: frames <-seq(from ,to , by= ). Default is NULL (all the frames of the trajectory are taken into account).  
 }
}


\value{
  Returns a list of class 'structure' with the following elements containing information on the sequence, structure, trajectory and side chain dihedral angles (values and names) of the protein during the simulation:
  \item{pdb}{
  an object of class 'pdb' created by the \code{read.pdb}function from the \code{bio3d} package
  }
  \item{dcd}{
  A numeric matrix of xyz coordinates with a frame per row and a Cartesian coordinate per column. Created by the \code{read.dcd}function from the \code{bio3d} package  
  }
  \item{xyz}{
  A numeric matrix of xyz coordinates with a frame per row and a Cartesian coordinate per column. For each frame, the protein coordinates have been fitted on the pdb structure using the \code{fit.xyz} from the \code{bio3d} package  
  }
  \item{tor}{
  A numeric matrix of side chain dihedral angles with a frame per row and a dihedral angle per column. Contains only side chain dihedral angles. Created by the \code{xyz2tor} function from the \code{bio3d} package  
  }
  \item{tor.names}{
  a character vector with the names of all side chain chi dihedral angles. They are written as "M.chiN" where M is the residue number and N the dihedral angle chi (chi1, chi2,...). Alanine and Glycine residues which do not have side chain dihedral angles are omitted. 
  }
  \item{tor.resno}{
  a character vector with the residue number M of all side chain chi dihedral angles. 
  }
  \item{tor.angle}{
  a character vector with the dihedral angle (chiN) of all side chain chi dihedral angles. 
  }
  \item{nb_torsions}{
  a numeric value indicating the total number of dihedral angles 
  }
  \item{prot.seq}{
  a character vector with the sequence of the protein 
  }
  \item{nb_residues}{
  a numeric value indicating the number of residues in the protein
  }
  \item{nb_frames}{
  a numeric value indicating the total number of selected frames 
  }
  \item{frames}{
  a numeric vector indicating the selected frames 
  }



}

\author{
  Bruck TADDESE, Antoine GARNIER and Marie CHABBERT
}

\examples{
  #Reading pdb and dcd files
  pdb <- system.file("rotamer/toy_coordinates.pdb", package= "Bios2cor")
  trj <- system.file("rotamer/toy_dynamics.dcd", package= "Bios2cor")
  
  #Creating dynamic_structure object for selected frames  
  wanted_frames <- seq(from = 1, to = 40, by = 2)
  dynamic_structure <- dynamic_structure(pdb, trj, wanted_frames)

  #Creating dynamic_structure object for all the frames  
  #dynamic_structure <- dynamic_structure(pdb, trj)
}

 

\name{angles.plot}

\Rdversion{1.1}

\alias{angles.plot}

\title{
  plots the time evolution of the dihedral angles in top pairs
}

\description{
  Given an object of class 'structure' and names of dihedral angles, creates plots of the dihedral angles as a function of frames in a pdf file. 
}

\usage{
  angles.plot(dynamic_structure, filepathroot, angles)
}

\arguments{
 \item{dynamic_structure}{
   Dihedral angle structure, result of the \code{dynamic_structure} function
 }
 \item{angles}{
   A vector containing the names of the dihedral angles to be visualized.Default is NULL (all the torsional angles of the dynamic_structure object are taken into account). 
 }
 \item{filepathroot}{
   Root of the full path name for the output file. If NULL, an output file "ANGLES.pdf" is created in tempdir(). If not NULL, a "_ANGLES.pdf" extention is added to the filepathroot.
 }
}

\details{
  The object of class 'structure' contains the side chain dihedral angles (between -180 and 180) for each residue in the protein, for each frame of the molecular simulations. This function allows visualisation of the evolution of selected angles. 
}

\value{
  returns a pdf file containing the plots of the frame dependance of each element included in argument \code{angles}.
}

\author{
  Antoine GARNIER and Marie CHABBERT
}

\examples{  
  #Indicating file path for output files
  out <- tempdir()
  file <- file.path(out,"test_dyn1") 

  #Reading pdb and dcd files
  pdb <- system.file("rotamer/toy_coordinates.pdb", package= "Bios2cor")
  trj <- system.file("rotamer/toy_dynamics.dcd", package= "Bios2cor")
  
  #Creating dynamic_structure object for wanted frames
  wanted_frames <- seq(from= 1, to= 40, by= 2)
  dynamic_structure <- dynamic_structure(pdb, trj, wanted_frames)

  #Calculating circular correlation between dihedral angles of selected residues 
  wanted_residues <- c("H","N","Q","F","Y","W")
  dihed_corr <- dynamic_circular(dynamic_structure, wanted_residues)

  #Selecting a correlation matrix
  dihed_corr <- dihed_corr$Zscore
  
  #Selecting angles of interest (here from the "top_pairs_analysis" function)
  top_angles <- top_pairs_analysis(dihed_corr, top= 25, file)
  my_angles <- unlist(top_angles$positions)
  
  #Creating plots of the time evolution of the dihedral angles 
  evol_angles <- angles.plot(dynamic_structure, file, my_angles)

}

 

\name{import.fasta}
\Rdversion{1.1}
\alias{import.fasta}
\title{
Reads a file in FASTA format
}
\description{
 Reads a Multiple Sequence Alignment (MSA) file in FASTA format (.fasta or .fa extension).
}
\usage{
 import.fasta(file, aa.to.upper = TRUE, gap.to.dash = TRUE)
}
\arguments{
  \item{file}{
    a string of characters to indicate the name of the MSA file to be read.
  }
  \item{aa.to.upper}{
    a logical value indicating whether amino acids should be converted to upper case (TRUE) or not (FALSE). Default is TRUE. 
  }
  \item{gap.to.dash}{
    a logical value indicating whether the dot (.) and tilde (\eqn{\sim}) gap symbols should be converted
    to dash (-) character (TRUE) or not (FALSE). Default is TRUE.
  }
}

\details{
  Initially, FASTA (for FAST-ALL) was the input format of the FASTA program, used for protein comparison and searching in databases.
  Presently, FASTA format is a standard format for biological sequences.
  
  The FASTA formatted file of a single sequence displays:
  \itemize{
    \item a single-line description beginning with a greater-than (>) symbol. The following word is the identifier.
    \item followed by any number of lines, representing biological sequence.
  }
  For multiple alignments, the FASTA formatted sequences are concatenated to create a multiple
  FASTA format.
}

\value{
  A object of class 'align', which is a named list whose elements correspond to sequences, in the form of character vectors.
}

\references{
  Pearson WR and Lipman DJ (1988) Improved tools for biological sequence comparison.
  \emph{Proc Natl Acad Sci U S A} \bold{27}:2444-2448.
}

\note{
  The \code{import.fasta} function was developed for the \code{bios2mds} R package (Julien PELE [aut], Jean-Michel BECU [aut], Marie CHABBERT [cre]).
.
}

\author{
  Julien PELE
}


\examples{
  # Importing MSA file in FASTA format
  aln <- import.fasta(system.file("msa/toy2_align.fa", package = "Bios2cor"))
}

\name{dynamic_circular}

\Rdversion{1.1}

\alias{dynamic_circular}

\title{
  Circular correlation
}

\description{
  Calculates circular correlation/covariation scores between side chain dihedral angles during a molecular dynamics trajectory. 
}

\usage{
  dynamic_circular(
    dynamic_structure,
    res_selection=
      c("C","I","L","M","V","R","H","K","D","E","N","Q","F","Y","W","T","S","P")
  )
}

\arguments{
 \item{dynamic_structure}{
   Object of class 'structure' that is created by the \code{\link{dynamic_structure}} function
 }
 \item{res_selection}{
   List of amino acids that will be taken into account in the correlation/covariation matrix. By default, all the amino acids are taken into account except Gly and Ala, with no side chain dihedral angles.  
 }
}

\details{
  This function uses the \code{cor.circular} function from the \code{circular} package based on a circular version of the Pearson coeefficient. 
}

\value{
  returns a list of four elements which are numeric matrices containing (1) the correlation/covariation scores for each pair of rotamers (score), (2) the Z-scores for each pair of rotamers (Zscore), (3) the correlation/covariation scores for each pair of rotamers with zero values for autocorrelation (correlation within the same side chain) (score_noauto) and (4) the Z-scores calculated without autocorrelation pairs and zero values for autocorrelation pairs (Zscore_noauto). 
}

\author{
   Bruck TADESSE, Antoine GARNIER, and Marie CHABBERT 
}

\references{
  Circular Statistics, from ``Topics in circular Statistics'' (2001) S. Rao Jammalamadaka and A. SenGupta, World Scientific.
}


\examples{
  #Reading pdb and dcd files
  pdb <- system.file("rotamer/toy_coordinates.pdb", package= "Bios2cor")
  trj <- system.file("rotamer/toy_dynamics.dcd", package= "Bios2cor")

  #Creating dynamic_structure object for selected frames
  wanted_frames <- seq(from = 1, to = 40, by = 2)
  dynamic_structure <- dynamic_structure(pdb, trj, wanted_frames)

  #Computing circular correlation between dihedral angles of selected residues
  res_selection <- c("H","N","Q","F","Y","W")    
  dihed_corr <- dynamic_circular(dynamic_structure, res_selection)

  #Computing correlation between all dihedral angles 
  #dihed_corr <- dynamic_circular(dynamic_structure)

}
